﻿<####################################################################################################
SolidSCP - Simple server setup menu

v1.0    24th May 2016:    First release of the SolidCP Install Script
v1.0.1  15th June 2016    2nd release - added Exchange 2016 and Silent Installation to the script
v1.0.2  16th June 2016    3rd release - Fixes on Option 4 (removed [TAB] and replaced with [SPACE] so PowerShell doesn't try to Auto Complete
v1.0.3  22nd June 2016    4th release - Added domain membership checking as well as various bug fixes for UAC Hardening
v1.0.4  23nd June 2016    5th release - Added MariaDB Database Server as well as various fixes
v1.0.5  25th June 2016    6th release - New menu structure added for simplicity
v1.0.6  28th June 2016    7th release - Additional deployment options for Active Directory
v1.0.7  04th July 2016    8th release - Added XML function and improved the UAC hardening for the XML files in IIS for .NET

Written By Marc Banyard for the SolidCP Project (c) 2016 SolidCP

Copyright (c) 2016, SolidCP
SolidCP is distributed under the Creative Commons Share-alike license

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must  retain  the  above copyright notice, this
  list of conditions and the following disclaimer.

- Redistributions in binary form  must  reproduce the  above  copyright  notice,
  this list of conditions  and  the  following  disclaimer in  the documentation
  and/or other materials provided with the distribution.

- Neither the name of  SolidCP  nor the names of its contributors may be used to
  endorse or  promote  products  derived  from  this  software  without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT  NOT  LIMITED TO, THE IMPLIED
WARRANTIES  OF  MERCHANTABILITY   AND  FITNESS  FOR  A  PARTICULAR  PURPOSE  ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO,  PROCUREMENT  OF  SUBSTITUTE  GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED AND ON
ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT  LIABILITY,  OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

All Code provided as is and used at your own risk.
####################################################################################################>
Write-Host "
        ****************************************
        *                                      *
        *        Welcome to the SolidCP        *
        *          Automated Installer         *
        *                                      *
        *       Please be patient whilst       *
        *          the menu is loaded          *
        *                                      *
        ****************************************" -ForegroundColor Green
####################################################################################################
# Additional items used in the below functions, they are stored as variables for easy use later    #
####################################################################################################
$dIPV4 = ((Test-Connection $env:computername -count 1).IPv4address.IPAddressToString)       # Get IPV4 Address
$dIPV6 = ((Resolve-DnsName -Name $env:COMPUTERNAME -Type AAAA).IPAddress -notlike 'fe80:*') # Get IPV6 Address
$dDomainMember     = ((gwmi win32_computersystem).partofdomain -eq $true)                   # Check if machine is joined to a domain
$dFQDNthisMachine  = ([System.Net.Dns]::GetHostByName(($env:computerName)).HostName)        # Get the FQDN of this machine
$dRootDN           = (([ADSI]"LDAP://RootDSE").rootDomainNamingContext)                     # Get the Root Distinguished Name from Active Directory (i.e. DC=SolidCP,DC=com)
$dDomainName       = $env:USERDNSDOMAIN  # Store the Domain Name (if joined) as a variable to use later
$dComputerName     = $env:computername   # Store the Computer Name as a variable to use later
$dLoggedInUserName = $env:USERNAME       # Store the Logged On User Name as a variable to use later
####################################################################################################
####################################################################################################

# Editable features are below this line

$InstallSNMPchk    = $true                  # Enable or disable by $true|$false
$SNMPsysContact    = "Please Change Me"     # SNMP Contact Name
$SNMPsysLocation   = "Please Change Me"     # SNMP Server Location
$dMsSQLpassword    = ""                     # Enter the password for your MS SQL "sa" User
$dMySQLvMariaDB    = "MariaDB"              # Choose between "MySQL|MariaDB" for the Database Type
$dMariaMySQLpasswd = ""                     # Enter the password for your MariaDB or MySQL "root" User
$dExchangeOrgNme   = "HostedExchange"       # Set the Exchange Organisation Name
$dADhostedOUname   = "SolidCP"              # Set the Active Directory Organisational Unit Name
$dExchangeInstal   = "D:"                   # Set the installation directory of where your Exchange files are located (i.e. D: for the DVD)
$dSolidCPportalPortNumber = "80"            # SolidCP Web Portal - Port Number
$dSolidCPportalIPaddress  = $dIPV4          # SolidCP Web Portal - IP Address
$dSolidCPportalPassword   = "YourPassword"  # SolidCP Web Portal - serveradmin Password
$dSolidCPserverPassword   = "YourPassword"  # SolidCP Server - Password
$dSolidCPCloudPortlPasswd = "YourPassword"  # SolidCP Cloud Storage Server - Password
$dSolidCPEnterpriseSvrIP  = ""              # Enterprise Server IP Address (i.e. 192.168.1.1)
$dSolidCPEnterpriseSvrURL = ""              # Enterprise Server URL - FULL URL with Port (i.e. http://192.168.1.1:9002)
$dSCPdomainName           = ""              # FQDN used for Active Directory (i.e. hosted.solidcp.com)


# Editable features are above this line
####################################################################################################
#
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
#                                                                                                  #
# DO NOT EDIT ANYTHING BELOW THIS LINE                                                             #
#                                                                                                  #
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
# Language section, this is needed as Microsoft have different account names depending on language #
####################################################################################################
$dLocalAdministratorSID  = (Get-WmiObject -Query "SELECT SID FROM Win32_UserAccount WHERE LocalAccount = 'True'" | Select-Object -First 1 -ExpandProperty SID); # Local Administrator SID
$dDomainAdministratorSID = (Get-WmiObject -Query "SELECT SID FROM Win32_UserAccount WHERE LocalAccount = 'False'" | Select-Object -First 1 -ExpandProperty SID); # Domain Administrator SID
$dMachineSID             = ((Get-WmiObject -Query "SELECT SID FROM Win32_UserAccount WHERE LocalAccount = 'True'" | Select-Object -First 1 -ExpandProperty SID) -replace ".{4}$"); # Local Machine SID
$dDomainSID              = ((Get-WmiObject -Query "SELECT SID FROM Win32_UserAccount WHERE LocalAccount = 'False'" | Select-Object -First 1 -ExpandProperty SID) -replace ".{4}$"); # Domain SID
$dLoggedInLocally        = ( ((Get-WMIObject -class Win32_ComputerSystem | select username).username) -eq ("$env:COMPUTERNAME\$env:USERNAME") ) # Check if logged in locally
$dLangAdministratorName  = (([wmi]"Win32_SID.SID='$dLocalAdministratorSID'").AccountName); # Administrator
$dLangCreatorOwnerName   = (([wmi]"Win32_SID.SID='S-1-3-0'").AccountName);                 # CREATOR OWNER
$dLangSystemName         = (([wmi]"Win32_SID.SID='S-1-5-18'").AccountName);                # SYSTEM
$dLangLocalServiceName   = (([wmi]"Win32_SID.SID='S-1-5-19'").AccountName);                # LOCAL SERVICE
$dLangNetworkServiceName = (([wmi]"Win32_SID.SID='S-1-5-20'").AccountName);                # NETWORK SERVICE
$dLangAdministratorGroup = (([wmi]"Win32_SID.SID='S-1-5-32-544'").AccountName);            # Administrators
$dLangUsersGroup         = (([wmi]"Win32_SID.SID='S-1-5-32-545'").AccountName);            # Users
$dLangPreWin2000Group    = (([wmi]"Win32_SID.SID='S-1-5-32-554'").AccountName);            # Pre-Windows 2000 Compatible Access
if ($dDomainMember) { # Only do the following if the server is a member of a domain
	$dLangDomainAdminsGroup       = (([wmi]"Win32_SID.SID='$dDomainSID-512'").AccountName);          # Domain Admins
	$dLangDomainAdministratorName = (([wmi]"Win32_SID.SID='$dDomainAdministratorSID'").AccountName); # Administrator
	$dLangDomainEnterpriseAdmins  = (([wmi]"Win32_SID.SID='$dDomainSID-519'").AccountName);          # Enterprise Admins
}
####################################################################################################
# Main menu starts here
Function SolidCPmenu() {
	do {
		do {
		cls
			Write-Host "`n`tSolidCP Setup Menu`n" -ForegroundColor Magenta
			Write-Host "`t`tPlease select the type of server you are installing SolidCP onto`n" -ForegroundColor Cyan
	$menu = @"
			1. SolidCP Exterprise Server / Portal
			2. Microsoft DNS Server
			3. Microsoft Web Server
			4. Database Servers (MS SQL, MariaDB, MySQL)
			5. AwStats Server
			6. Active Directory (Domain Controllers)
			7. Email Servers
			8. Microsoft Remote Desktop Services

			X. Quit and exit
"@
	$menuOptions = '^[1-8x]+$'

			Write-Host $menu -ForegroundColor Cyan
			$choice = Read-Host "`n`tEnter Option From Above Menu"
			$ok = $choice -match $menuOptions
		
			if ( -not $ok) { Write-Host "Invalid selection" ; start-Sleep -Seconds 2 }
		} until ( $ok )

		switch -Regex ( $choice ) {
			"1" { dInstallConfirm "SolidCP Enterprise Server and Portal Requirements" $function:MainMenu_Option_1 }
			"2" { dInstallConfirm "Microsoft DNS Server" $function:MainMenu_Option_2 }
			"3" { MainMenu_Option_3 }
			"4" { MainMenu_Option_4 }
			"5" { dInstallConfirm "AwStats Server" $function:MainMenu_Option_5 ; $choice="X" }
			"6" { dInstallConfirm "Microsoft Active Directory Requirements" $function:MainMenu_Option_6 }
			"7" { MainMenu_Option_7 }
			"8" { MainMenu_Option_8 }
			"X" { MainMenu_Option_X }
		}
	} until ( $choice -match "X" )
}
# Main menu end

####################################################################################################################################################################################
####################################################################################################################################################################################
# Functions for the Main Menu Options are below here
####################################################################################################################################################################################
# Main Menu - Option 1 - Install SolidCP Enterpeise Server and Portal Server
Function MainMenu_Option_1() { # Function to call multiple functions to install SolidCP Enterprise Server and Portal
	CLS
	dCheckMsSQLpassword
	Write-Host "`nInstalling the required windows features for the 'SolidCP Enterprise Server and Portal'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforESandPortal
	HardenDotNETforIIS
	FixServer2012UAC
	InstallSQLsvr2014withTools $dMsSQLpassword
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Enterprise"
	InstallSOlidCPcomponent "Portal"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 2 - Install Microsoft DNS Server for SolidCP
Function MainMenu_Option_2() { # Function to call multiple functions to install Features for Microsoft DNS Server
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for a 'Microsoft DNS Server'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallDNSServer
	HardenDotNETforIIS
	FixServer2012UAC
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 3 - Present Sub Menu for Microsoft IIS Server deployment types
Function MainMenu_Option_3() { # Function to show the user a Sub Menu for the Microsoft IIS Server deployment options
	do {
		do {
		cls
			Write-Host "`n`tSolidCP Setup Menu - Microsoft Web Server`n" -ForegroundColor Magenta
			Write-Host "`t`tPlease select the Microsoft Web Server deployment you want to install for SolidCP`n`n" -ForegroundColor Cyan
			Write-Host "`t`t(IIS, FTP, PHP, PHP Manager, Python, Perl, MariaDB or MySQL, MS SQL)`n`n" -ForegroundColor Cyan
	$menu = @"
			A. Microsoft Web Server - Fully Automated
			B. Microsoft Web Server - (NO Database) Fully Automated
			C. Microsoft Web Server Prompted

			X. Exit back to Main Menu
"@
	$menuOptions = '^[a-cx]+$'

			Write-Host $menu -ForegroundColor Cyan
			$choice3 = Read-Host "`n`tEnter Option From Above Menu"
			$ok = $choice3 -match $menuOptions
		
			if ( -not $ok) { Write-Host "Invalid selection" ; start-Sleep -Seconds 2 }
		} until ( $ok )
		
		switch -Regex ( $choice3 ) {
			"A" { dInstallConfirm "IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL (Automated)" $function:MainMenu_Option_3_A }
			"B" { dInstallConfirm "IIS, FTP, PHP, Python, Perl - Without Database Servers (Automated)" $function:MainMenu_Option_3_B }
			"C" { dInstallConfirm "IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL (Prompted)" $function:MainMenu_Option_3_C }
		}
	} until ( $choice3 -match "X" )
}

####################################################################################################################################################################################
# Main Menu - Option 3 - Sub Menu A - Install Microsoft IIS Web Server for SolidCP
Function MainMenu_Option_3_A() { # Function to call multiple functions to install IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL
	CLS
	dCheckMsSQLpassword
	dCheckMariaMySQLpassword
	Write-Host "Installing the required windows features for a 'Microsoft Web Server' (IIS, FTP, PHP, Python, Perl, $dMySQLvMariaDB and MS SQL" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallFTPforHosting
	InstallWebPI_PHP
	InstallWebPI_Python
	InstallWebPI_SQLdriverPHP
	InstallWebPI_URLreWrite
	InstallWebPI_WebDeploy
	InstallActivePerl
	InstallMariaDB_MySQL $dMariaMySQLpasswd
	InstallSQLsvr2014withTools $dMsSQLpassword
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 3 - Sub Menu B - Install Microsoft IIS Web Server without Database Servers for SolidCP
Function MainMenu_Option_3_B() { # Function to call multiple functions to install IIS, FTP, PHP, Python, Perl
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for a 'Microsoft Web Server' (IIS, FTP, PHP, Python, Perl)" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallFTPforHosting
	InstallWebPI_PHP
	InstallWebPI_Python
	InstallWebPI_SQLdriverPHP
	InstallWebPI_URLreWrite
	InstallWebPI_WebDeploy
	InstallActivePerl
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 3 - Sub Menu C - Install Microsoft IIS Web Server for SolidCP - Prompted
Function MainMenu_Option_3_C() { # Function to call multiple functions to install Features for IIS, FTP, PHP, Python, Perl, MySQL and MS SQL
	CLS
	dCheckMsSQLpassword
	dCheckMariaMySQLpassword
	Write-Host "Installing the required windows features for a 'Microsoft Web Server - Prompted'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	dInstallConfirm "Microsoft FTP Server" $function:InstallFTPforHosting                           # Prompt to install Microsoft FTP Server
	dInstallConfirm "PHP for IIS" $function:InstallWebPI_PHP                                        # Prompt to install PHP for IIS
	dInstallConfirm "Python v3.4" $function:InstallWebPI_Python                                     # Prompt to install Python v3.4
	dInstallConfirm "SQL Server Driver for PHP" $function:InstallWebPI_SQLdriverPHP                 # Prompt to install SQL Server Driver for PHP
	dInstallConfirm "URL Rewrite Module" $function:InstallWebPI_URLreWrite                          # Prompt to install URL Rewrite Module
	dInstallConfirm "Web Deploy module" $function:InstallWebPI_WebDeploy                            # Prompt to install Web Deploy Module
	dInstallConfirm "Active Perl v5.22.1" $function:InstallActivePerl                               # Prompt to install Active Perl v5.22.1
	dInstallConfirm "$dMySQLvMariaDB Database Server" $function:InstallMariaDB_MySQL                # Prompt to install MariaDB or MySQL Database Server
	dInstallConfirm "Microsoft SQL Server 2014 Express" $function:InstallSQLsvr2014noTools          # Prompt to install Microsoft SQL Server 2014 Express
	dInstallConfirm "SQL Server 2014 Management Tools" $function:InstallSQLsvr2014mgmtTools         # Prompt to install Microsoft SQL Server 2014 Express Management Tools
	dInstallConfirm "Disable Password Complexity (Local)" $function:InstallDisablePasswdComplex     # Prompt to disable local password complexity
	dInstallConfirm "SolidCP Installer Application" $function:InstallSOlidCPInstaller               # Prompt to download and install the SOlidCP Installation Application
	InstallSOlidCPcomponent "Server"                                                                # Install the Server component ONLY of the installer is installed
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 4 - Present Sub Menu for Database Server deployment types
Function MainMenu_Option_4() { # Function to show the user a Sub Menu for the Database Server deployment options
	do {
		do {
		cls
			Write-Host "`n`tSolidCP Setup Menu - Database Server`n" -ForegroundColor Magenta
			Write-Host "`t`tPlease select the Database Server deployment you want to install for SolidCP`n`n" -ForegroundColor Cyan
	$menu = @"
			A. FULL Database Server - (Microsoft SQL Server 2014, MariaDB or MySQL)
			B. Microsoft SQL Server 2014 Express With Tools
			C. Microsoft SQL Server 2014 Express Database ONLY (No Tools)
			D. MariaDB Server or MySQL Server

			X. Exit back to Main Menu
"@
	$menuOptions = '^[a-dx]+$'

			Write-Host $menu -ForegroundColor Cyan
			$choice4 = Read-Host "`n`tEnter Option From Above Menu"
			$ok = $choice4 -match $menuOptions
		
			if ( -not $ok) { Write-Host "Invalid selection" ; start-Sleep -Seconds 2 }
		} until ( $ok )
		
		switch -Regex ( $choice4 ) {
			"A" { dInstallConfirm "FULL Database Server - (Microsoft SQL Server 2014, MariaDB or MySQL)" $function:MainMenu_Option_4_A }
			"B" { dInstallConfirm "Microsoft SQL Server 2014 Express With Tools" $function:MainMenu_Option_4_B }
			"C" { dInstallConfirm "Microsoft SQL Server 2014 Express Database ONLY (No Tools)" $function:MainMenu_Option_4_C }
			"D" { dInstallConfirm "MariaDB Server or MySQL Server" $function:MainMenu_Option_4_D }
		}
	} until ( $choice4 -match "X" )
}

####################################################################################################################################################################################
# Main Menu - Option 4 - Sub Menu A - Install Database Server for SolidCP
Function MainMenu_Option_4_A() { # Function to call multiple functions to install IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL
	CLS
	dCheckMsSQLpassword
	dCheckMariaMySQLpassword
	Write-Host "Installing the FULL Database Server - (Microsoft SQL Server 2014, $dMySQLvMariaDB Database Server)" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallMariaDB_MySQL $dMariaMySQLpasswd
	InstallSQLsvr2014withTools $dMsSQLpassword
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 4 - Sub Menu B - Install Database Server for SolidCP
Function MainMenu_Option_4_B() { # Function to call multiple functions to install IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL
	CLS
	dCheckMsSQLpassword
	Write-Host "Installing the Microsoft SQL Server 2014 Express Database ONLY (No Tools))" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallSQLsvr2014noTools $dMsSQLpassword
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 4 - Sub Menu C - Install Database Server for SolidCP
Function MainMenu_Option_4_C() { # Function to call multiple functions to install IIS, FTP, PHP, Python, Perl, MariaDB or MySQL and MS SQL
	CLS
	dCheckMsSQLpassword
	Write-Host "Installing the Microsoft SQL Server 2014 Express with Tools)" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallSQLsvr2014withTools $dMsSQLpassword
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 4 - Sub Menu D - Install MariaDB or MySQL Database Server for SolidCP
Function MainMenu_Option_4_D() { # Function to call multiple functions to install MariaDB or MySQL
	CLS
	dCheckMariaMySQLpassword
	Write-Host "Installing the $dMySQLvMariaDB Database Server)" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallIISforHosting
	HardenDotNETforIIS
	FixServer2012UAC
	InstallMariaDB_MySQL $dMariaMySQLpasswd
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 5 - Install AwStats Server for SolidCP
Function MainMenu_Option_5() { # Function to call multiple functions to install AwStats
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for the 'AwStats Server'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallAwStats
	HardenDotNETforIIS
	FixServer2012UAC
	InstallWebPI_PHP
	InstallWebPI_Python
	InstallActivePerl
	InstallDisablePasswdComplex
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 6 - Install Microsoft Active Directory
Function MainMenu_Option_6() { # Function to call multiple functions to install Active Directory Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n`n`n`n`n"  # This brings the text down by 12 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Active Directory (Domain Controllers)'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallActiveDirectory
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Present Sub Menu for Email Server deployment types
Function MainMenu_Option_7() { # Function to show the user a Sub Menu for the Email Server deployment options
	do {
		do {
		cls
			Write-Host "`n`tSolidCP Setup Menu - Email Servers`n" -ForegroundColor Magenta
			Write-Host "`t`tPlease select the Email Server deployment you want to install for SolidCP`n`n" -ForegroundColor Cyan
	$menu = @"
			A. Microsoft Exchange Server 2016 - CAS & Mailbox Roles
			B. Microsoft Exchange Server 2016 - File Share Witness - DAG ONLY
			C. Microsoft Exchange Server 2013 - Mailbox Server ONLY
			D. Microsoft Exchange Server 2013 - CAS Server ONLY
			E. MailEnable Email Server (POP3 & SMTP)

			W. Microsoft Exchange - Fix Failed Database Content Indexes

			X. Exit back to Main Menu
"@
	$menuOptions = '^[a-dW-x]+$'

			Write-Host $menu -ForegroundColor Cyan
			$choice7 = Read-Host "`n`tEnter Option From Above Menu"
			$ok = $choice7 -match $menuOptions
		
			if ( -not $ok) { Write-Host "Invalid selection" ; start-Sleep -Seconds 2 }
		} until ( $ok )
		
		switch -Regex ( $choice7 ) {
			"A" { dInstallConfirmDomain "Microsoft Exchange Server 2016 - CAS & Mailbox Roles" $function:MainMenu_Option_7_A }
			"B" { dInstallConfirmDomain "Microsoft Exchange Server 2016 - File Share Witness - DAG ONLY" $function:MainMenu_Option_7_B }
			"C" { dInstallConfirmDomain "Microsoft Exchange Server 2013 - Mailbox Server ONLY" $function:MainMenu_Option_7_C }
			"D" { dInstallConfirmDomain "Microsoft Exchange Server 2013 - CAS Server ONLY" $function:MainMenu_Option_7_D }
			"E" { dInstallConfirm "MailEnable Email Server" $function:MainMenu_Option_7_E }
			"V" { dInstallConfirmDomain "Microsoft Exchange - Post Upgrade Fixes" $function:MainMenu_Option_7_V }
			"W" { dInstallConfirmDomain "Microsoft Exchange - Fix Failed Database Content Indexes" $function:MainMenu_Option_7_W }
		}
	} until ( $choice7 -match "X" )
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu A - Install Microsoft Exchange 2016 for SolidCP
Function MainMenu_Option_7_A() { # Function to call multiple functions to install Microsoft Exchange 2016 MBX Role Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Exchange 2016 Server Role (Mailbox and CAS)'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	Exchange2016_PreCheck
	InstallCommonFeatures
	InstallIISforSolidCP
	ExchangeMBX2016
	dInstallConfirm "SolidCP Installer Application" $function:InstallSOlidCPInstaller  # Prompt to download and install the SOlidCP Installation Application
	InstallSOlidCPcomponent "Server"                                                   # Install the Server component ONLY of the installer is installed
	Exchange2016_PostCheck
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu B - Install Microsoft Exchange File Share Witness (DAG ONLY Role) for SolidCP
Function MainMenu_Option_7_B() { # Function to call multiple functions to install Microsoft Exchange File Share Witness - DAG ONLY Role Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Exchange (File Share Witness - DAG ONLY)'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallExchangeFSW
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu C - Install Microsoft Exchange 2013 CAS Role for SolidCP
Function MainMenu_Option_7_C() { # Function to call multiple functions to install Microsoft Exchange CAS Role Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Exchange (CAS Server Role)'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallExchangeCAS
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu D - Install Microsoft Exchange 2013 Mailbox Role for SolidCP
Function MainMenu_Option_7_D() { # Function to call multiple functions to install Microsoft Exchange Mailbox Role Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Exchange (Mailbox Server Role)'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallExchangeMBX
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu E - Install MailEnable Email Server for SolidCP
Function MainMenu_Option_7_D() { # Function to call multiple functions to install Install MailEnable Email Server
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Install MailEnable Email Server'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallIISforSolidCP
	InstallMailEnable
	InstallSOlidCPInstaller
	InstallSOlidCPcomponent "Server"
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu V - Chec Microsoft Exchange 2016 after Service Pack or Cumulative Updates
Function MainMenu_Option_7_V() { # Function to check the Microsoft Exchange 2016 Server after doing Exchange Updates
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Checking 'Microsoft Exchange 2016 Server' after Windows or Exchange Updates" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	Exchange2016_PreCheck
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 7 - Sub Menu W - Microsoft Exchange - Fix Failed Database Content Indexes for SolidCP
Function MainMenu_Option_7_W() { # Function to call multiple functions to Microsoft Exchange - Fix Failed Database Content Indexes
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Microsoft Exchange - Fixing the Failed Database Content Indexes" -ForegroundColor Magenta
	Write-Host "`tNothing done, this will be added soon" -ForegroundColor Green ; start-Sleep -Seconds 2
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 8 - Present Sub Menu for Microsoft Remote Desktop Services deployment types
Function MainMenu_Option_8() { # Function to show the user a Sub Menu for the Microsoft Exchange Server deployment options
	do {
		do {
		cls
			Write-Host "`n`tSolidCP Setup Menu - Exchange Server`n" -ForegroundColor Magenta
			Write-Host "`t`tPlease select the Microsoft Exchange Server deployment you want to install for SolidCP`n`n" -ForegroundColor Cyan
	$menu = @"
			A. Microsoft Remote Desktop Services - Host Server
			B. Microsoft Remote Desktop Services - Licencing Server

			X. Exit back to Main Menu
"@
	$menuOptions = '^[a-bx]+$'

			Write-Host $menu -ForegroundColor Cyan
			$choice8 = Read-Host "`n`tEnter Option From Above Menu"
			$ok = $choice8 -match $menuOptions
		
			if ( -not $ok) { Write-Host "Invalid selection" ; start-Sleep -Seconds 2 }
		} until ( $ok )
		
		switch -Regex ( $choice8 ) {
			"A" { dInstallConfirm "Microsoft Remote Desktop Services - Host Server" $function:MainMenu_Option_8_A }
			"B" { dInstallConfirm "Microsoft Remote Desktop Services - Licencing Server" $function:MainMenu_Option_8_B }
		}
	} until ( $choice8 -match "X" )
}

####################################################################################################################################################################################
# Main Menu - Option 8 - Sub Menu A - Install Microsoft Remote Desktop Services Host Server Requirements for SolidCP
Function MainMenu_Option_8_A() { # Function to call multiple functions to install Microsoft Remote Desktop Services Host Server Requirements
	CLS
	Write-Host "`n`n`n`n`n`n`n`n"  # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
	Write-Host "Installing the required windows features for 'Microsoft Remote Desktop Services - Host Server'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallRDShostServer
	InstallSQLsvr2012NativeClnt
	HardenDotNETforIIS
	FixServer2012UAC
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option 8 - Sub Menu B - Install Microsoft Remote Desktop Services Licencing Server Requirements for SolidCP
Function MainMenu_Option_8_B() { # Function to call multiple functions to install Microsoft Remote Desktop Services Licencing Server Requirements
	CLS
	dCheckMsSQLpassword
	Write-Host "Installing the required windows features for 'Microsoft Remote Desktop Services - Licencing Server'" -ForegroundColor Magenta
	Write-Host "`tStarting server setup for SolidCP`n" -ForegroundColor Gray
	InstallCommonFeatures
	InstallSQLsvr2014withTools $dMsSQLpassword
	HardenDotNETforIIS
	FixServer2012UAC
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Main Menu - Option X - Exit the SolidCP Menu
Function MainMenu_Option_X() { # Function to Exit this menu
	CLS
	Write-Host "`n`t ******************************" -ForegroundColor Green
	Write-Host "`t *                            *" -ForegroundColor Green
	Write-Host "`t *          Quitting          *" -ForegroundColor Green
	Write-Host "`t *                            *" -ForegroundColor Green
	Write-Host "`t *    NO Changes were made    *" -ForegroundColor Green
	Write-Host "`t *                            *" -ForegroundColor Green
	Write-Host "`t ******************************" -ForegroundColor Green
	dPressAnyKeyToExit
}

####################################################################################################################################################################################
# Functions for the Main Menu Options are above here
####################################################################################################################################################################################
####################################################################################################################################################################################



########################################################################################################################################################################################################
# General Functions start below here #
######################################


####################################################################################################################################################################################
Function dInstallConfirm($dInstallName, $sFunctionToRun)        # Function to prompt for confirmation to install application called from a function
{ # Usage - dInstallConfirm "SQL Server" $function:dTestScript
	CLS
	$choice = ""
	while ($choice -notmatch "[y|n]") { CLS ; $choice = read-host "`n Do you want to install $dInstallName`? (Y/N)" }
	if ($choice -eq "y") {
		$($sFunctionToRun.Invoke())
		Write-Host "`n`t$dInstallName has been installed" -ForegroundColor Green ; start-Sleep -Seconds 2
	} else {
		write-host "`n`t$dInstallName installation cancelled by user" -ForegroundColor Red    ; start-Sleep -Seconds 2
	}
}


####################################################################################################################################################################################
Function dInstallConfirmDomain($dInstallName, $sFunctionToRun)  # Function to prompt for confirmation to install application called from a function if Domain Checking is required
{ # Usage - dInstallConfirmDomain "Exchange 2016" $function:dTestScript
	CLS
	if (!$dDomainMember) {
		Write-Host "`n`t *************************************************" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *   This machine is NOT a member of a domain    *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *   This server MUST be joined to a domain and  *" -ForegroundColor Yellow
		Write-Host "`t *    you need to login with a domain account    *" -ForegroundColor Yellow
		Write-Host "`t *  that is a member of the Domain Admins group  *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *    Please join this server to a domain and    *" -ForegroundColor Yellow
		Write-Host "`t *      log back on with an account that is      *" -ForegroundColor Yellow
		Write-Host "`t *      a member of the Domain Admins group      *" -ForegroundColor Yellow
		Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *************************************************" -ForegroundColor Yellow
	}elseif ( ($dDomainMember) -and (!(CheckGroupMembers "$dLangDomainAdminsGroup" "$dLoggedInUserName" "Domain")) ) {
		Write-Host "`n`t *************************************************" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *     This machine is a member of a domain      *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *    You need to login with a domain account    *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *  Please log back on with an account that is   *" -ForegroundColor Yellow
		Write-Host "`t *      a member of the Domain Admins group      *" -ForegroundColor Yellow
		Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *************************************************" -ForegroundColor Yellow
	}else{
		$choice = ""
		while ($choice -notmatch "[y|n]") { CLS ; $choice = read-host "`n Do you want to install $dInstallName`? (Y/N)" }
		if ($choice -eq "y") {
			$($sFunctionToRun.Invoke())
			Write-Host "`n`t$dInstallName has been installed" -ForegroundColor Green ; start-Sleep -Seconds 2
		} else {
			write-host "`n`t$dInstallName installation cancelled by user" -ForegroundColor Red ; start-Sleep -Seconds 2
		}
	}
}


####################################################################################################################################################################################
Function dRebootConfirm()                                   # Function to prompt for confirmation to reboot this machine
{
	$choice = ""
	while ($choice -notmatch "[y|n]") { $choice = read-host "`n Do you want to reboot this machine`? (Y/N)" }
	if ($choice -eq "y") {
		Write-Host "`n`tRebooting this machine" -ForegroundColor Yellow ; start-Sleep -Seconds 2
		Restart-Computer
	} else {
		write-host "`n`tYou MUST Reboot this machine" -ForegroundColor Red ; start-Sleep -Seconds 2
	}
}


####################################################################################################################################################################################
Function dCheckMsSQLpassword()                              # Function to prompt for confirmation to view the password for the Microsoft SQL Server that have just been entered
{
	if (!$dMsSQLpassword) { # Ask for a password if none has been specified for the MariaDB or MySQL installation
		Write-Host "`t*************************************************" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*          YOUR ATTENTION IS REQUIRED!          *" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*     You need to enter a password for your     *" -ForegroundColor Red
		Write-Host "`t*              Microsft SQL Server              *" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*************************************************" -ForegroundColor Red
		$responseMsSQL = Read-host "`n`tEnter the SA password you want to use for your Microsoft SQL Server" -AsSecureString
		$script:dMsSQLpassword = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($responseMsSQL))

		# if a password has been entered ask the user if they want to view it
		$choiceCheckMsSQLpassword = ""
		while ($choiceCheckMsSQLpassword -notmatch "[y|n]") {
			$choiceCheckMsSQLpassword = read-host "`tDo you want to view the SA Password you have entered for your Microsoft SQL Server? (Y/N)"
		}
		if ($choiceCheckMsSQLpassword -eq "y") {
			Write-Host "`n The SA Password entered for your Microsoft SQL Server was - $dMsSQLpassword" -ForegroundColor Green
			dPressAnyKeyToContinue
			CLS
		}
	}
	Write-Host "`n`n`n`n`n`n`n`n" # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
}


####################################################################################################################################################################################
Function dCheckMariaMySQLpassword()                         # Function to check the password for the MariaDB or MySQL Server that have just been entered, if not ask for it
{
	if (!$dMySQLvMariaDB) { # Check if MariaDB or MySQL has been specified, if not ask which one needs to be installed
		$choiceCheckMariaMySQL = ""
		while ($choiceCheckMariaMySQL -notmatch "[1|2]") { # Present the user with a menu to choose MariaDB or MySQL
			CLS
			Write-Host "`n`tPlease select the Database Type you want to install`n" -ForegroundColor Cyan
			Write-Host "`t 1. MariaDB Database Server" -ForegroundColor Cyan
			Write-Host "`t 2. MySQL Database Server" -ForegroundColor Cyan
			$choiceCheckMariaMySQL = read-host "`n`tEnter Option From Above Menu" 
		}
		if ($choiceCheckMariaMySQL -eq "1") { # Options for MariaDB
			$script:dMySQLvMariaDB = "MariaDB"
			Write-Host "`t The MariaDB Database will be installed on this machine" -ForegroundColor Green
		}
		if ($choiceCheckMariaMySQL -eq "2") { # Options for MariaDB
			$script:dMySQLvMariaDB = "MySQL"
			Write-Host "`t The MySQL Database will be installed on this machine" -ForegroundColor Green
		}
	}

	if (!$dMariaMySQLpasswd) { # Ask for a password if none has been specified for the MariaDB or MySQL installation
		if     ($dMySQLvMariaDB -eq "MariaDB") { $dMariaMySQLTxt = " MariaDB Server" } # Set Attention text for MariaDB
		elseif ($dMySQLvMariaDB -eq "MySQL"  ) { $dMariaMySQLTxt = "  MySQL Server " } # Set Attention text for MySQL
		Write-Host "`t*************************************************" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*          YOUR ATTENTION IS REQUIRED!          *" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*     You need to enter a password for your     *" -ForegroundColor Red
		Write-Host "`t*                $dMariaMySQLTxt                *" -ForegroundColor Red
		Write-Host "`t*                                               *" -ForegroundColor Red
		Write-Host "`t*************************************************" -ForegroundColor Red
		$responseMySQL = Read-host "`n`tEnter the ROOT password you want to use for your $dMySQLvMariaDB Server" -AsSecureString
		$script:dMariaMySQLpasswd = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($responseMySQL))

		# if a password has been entered ask the user if they want to view it
		$choiceCheckMariaMySQLpassword = ""
		while ($choiceCheckMariaMySQLpassword -notmatch "[y|n]") {
			$choiceCheckMariaMySQLpassword = read-host "`tDo you want to view the ROOT Password you have just entered for your $dMySQLvMariaDB Server? (Y/N)"
		}
		if ($choiceCheckMariaMySQLpassword -eq "y") {
			Write-Host "`t The ROOT Password entered for your $dMySQLvMariaDB Server was - $dMariaMySQLpasswd" -ForegroundColor Green
			dPressAnyKeyToContinue
			CLS
		}
	}
	Write-Host "`n`n`n`n`n`n`n`n" # This brings the text down by 8 lines to allow fo the Windows Features installation overlay
}


####################################################################################################################################################################################
Function dPressAnyKeyToExit()                               # Function to press any key to exit
{
	if ($psISE) { # Check if running Powershell ISE
		Add-Type -AssemblyName System.Windows.Forms
		[System.Windows.Forms.MessageBox]::Show("Press any key to exit")
		exit
	}else{
		Write-Host "`n`tPress any key to exit..." -ForegroundColor Yellow
		$x = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
		exit
	}
}


####################################################################################################################################################################################
Function dPressAnyKeyToContinue()                           # Function to press any key to exit
{
	if ($psISE) { # Check if running Powershell ISE
		Add-Type -AssemblyName System.Windows.Forms
		[System.Windows.Forms.MessageBox]::Show("Press any key to continue")
	}else{
		Write-Host "`n`tPress any key to continue..." -ForegroundColor Yellow
		$x = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
	}
}


####################################################################################################################################################################################
Function InstallCommonFeatures()                            # Function for common features on the servers
{
	# Enable Windows Updates (Notify ONLY) on the machine
	Write-Host "`tEnabling Windows Updates on this machine" -ForegroundColor Cyan
	Set-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update" -Name IsOOBEInProgress -value 0 -Type DWord
	Set-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update" -Name AUOptions -value 2 -Type DWord
	Set-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update" -Name IncludeRecommendedUpdates -value 1 -Type DWord
	Set-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update" -Name CachedAUOptions -value 2 -Type DWord
	(new-object -c "microsoft.update.servicemanager").addservice2("7971f918-a847-4430-9279-4a52d1efe18d",7,"") | Out-Null
	invoke-command{net stop wuauserv} | Out-Null
	invoke-command{net start wuauserv} | Out-Null
	# Start installing the Common Features required on the machine
	Write-Host "`tInstalling the common features required for SolidCP" -ForegroundColor Cyan
	(Import-Module ServerManager) | Out-Null
	# Install .NET Framework 3.5 on the server
	(Add-WindowsFeature -Name Net-Framework-Core) | Out-Null
	# Install PowerShell (current) as well as the older v2 for compatability
	(Add-WindowsFeature -Name PowerShell, PowerShell-V2) | Out-Null
	# Install SNMP and configure it on this machine as long as the $InstallSNMPchk is set as true
	if ($InstallSNMPchk) {
		# Install and Configure SNMP on the server so we can monitor it
		(Add-WindowsFeature -Name SNMP-Service -IncludeAllSubFeature -IncludeManagementTools) | Out-Null
		if (Get-Itemproperty "HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\PermittedManagers" | Select-Object -ExpandProperty 1 -ErrorAction Stop) {
			Remove-Itemproperty -Path HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\PermittedManagers -Name 1
		}
		Set-ItemProperty -Path HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\RFC1156Agent -Name sysContact -Value "$SNMPsysContact"
		Set-ItemProperty -Path HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\RFC1156Agent -Name sysLocation -Value "$SNMPsysLocation"
		Set-ItemProperty -Path HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\RFC1156Agent -Name sysServices -Value 79 -Type DWord
		Set-ItemProperty -Path HKLM:\SYSTEM\CurrentControlSet\Services\SNMP\Parameters\ValidCommunities -Name public -Value 4 -Type DWord
	}
	# Set the Start Button to display All Apps rather than the silly start menu
	Set-ItemProperty -Path HKCU:\Software\Microsoft\Windows\CurrentVersion\Explorer\StartPage -Name MakeAllAppsDefault -Value 1 -Type DWord
	# Install the Telnet Client and TFTP Client as we might need them at a later date
	(Add-WindowsFeature -Name Telnet-Client) | Out-Null
	(Add-WindowsFeature -Name TFTP-Client) | Out-Null
	# Disable IE Enhanced Security Configuration for Administrators of this server only
	Set-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Active Setup\Installed Components\{A509B1A7-37EF-4b3f-8CFC-4F3A74704073}" -Name IsInstalled -Value 0 -Type DWord
	Stop-Process -Name Explorer
}



####################################################################################################################################################################################
Function InstallIISforSolidCP()         # Function to install SolidCP IIS (Basic Requirements)
{
	Write-Host "`tInstalling the features required for SolidCP to run on IIS Server" -ForegroundColor Cyan
	# Install the features required by SolidCP Server to run on the machine, this is the minimum requirements for SolidCP to be installed on the server.
	(Add-WindowsFeature -Name Web-Server, Web-WebServer, Web-Common-Http, Web-Default-Doc, Web-Dir-Browsing, Web-Http-Errors, Web-Static-Content, Web-Http-Redirect, Web-Health, Web-Http-Logging, Web-Log-Libraries, Web-Http-Tracing, Web-Performance, Web-Stat-Compression, Web-Security, Web-Filtering, Web-Client-Auth, Web-Windows-Auth, Web-App-Dev, Web-Net-Ext, Web-Net-Ext45, Web-Asp-Net, Web-Asp-Net45, Web-ISAPI-Ext, Web-ISAPI-Filter, Web-Mgmt-Tools, Web-Mgmt-Console, Web-Scripting-Tools) | Out-Null
}


####################################################################################################################################################################################
Function InstallIISforHosting()         # Function to install additional IIS features for Web Server (Customer Hosting Requirements)
{
	Write-Host "`tInstalling the features required for Microsoft IIS Server (Web Hosting)" -ForegroundColor Cyan
	# Install the additional features required for a customer web server or shared hosting server that will be public facing
	(Add-WindowsFeature -Name Web-ODBC-Logging, Web-Request-Monitor, Web-Basic-Auth, Web-IP-Security, Web-ASP, Web-CGI, Web-Mgmt-Compat, Web-Metabase, Web-Lgcy-Mgmt-Console, Web-Lgcy-Scripting, Web-WMI, SMTP-Server) | Out-Null
	# Configure all logging options to be enabled for websites in IIS
	Set-WebConfigurationProperty -Filter System.Applicationhost/Sites/SiteDefaults/logfile -Name LogExtFileFlags -Value "Date,Time,ClientIP,UserName,SiteName,ComputerName,ServerIP,Method,UriStem,UriQuery,HttpStatus,Win32Status,BytesSent,BytesRecv,TimeTaken,ServerPort,UserAgent,Cookie,Referer,ProtocolVersion,Host,HttpSubStatus"
}


####################################################################################################################################################################################
Function InstallFTPforHosting()         # Function to install additional IIS features for FTP Server (Customer Hosting Requirements)
{
	Write-Host "`tInstalling the features required for Microsoft FTP Server" -ForegroundColor Cyan
	# Install the additional features required for a customer FTP server or shared hosting server that will be public facing
	(Add-WindowsFeature -Name Web-Ftp-Server, Web-Ftp-Service) | Out-Null
	Write-Host "`tConfiguring the FTP Server on this machine" -ForegroundColor Cyan
	# Check if the FTP Server is installed on this machine, if it check to make sure the Default FTP Site is not setup, then set it up corectly for SolidCP
	If ( ((Get-WindowsFeature *Web-Ftp-Service*).Installed[0]) -And !(Test-Path "IIS:\Sites\Default FTP Site" -pathType container) ) {
		# Create a local group called "SCPFtpUsers" on the local web server unless it already exists
		CreateLocalUserOrGroup "SCPFtpUsers" "SolidCP FTP Users Group  ********* DO NOT DELETE *********" "Group"
		# Create the "ftproot" folder in "C:\inetpub" if it doesnt exist
		if(!(Test-Path "$env:SystemDrive\inetpub\ftproot")) {New-Item "$env:SystemDrive\inetpub\ftproot" -itemType directory}
		# Grant permissions for the "C:\inetpub\ftproot" folder to the SCPFtpUsers Group (Read ONLY Access)
		SetAccessToFolder "C:\inetpub\ftproot" "SCPFtpUsers" "Read" "Allow" "Local"
		# Create the FTP Site for SolidCP only if it does not alreayd exist
		if (!(Test-Path "IIS:\Sites\Default FTP Site" -pathType container)) { # Setup the SolifCP FTP Site only if it doesn't already exist
			Import-Module WebAdministration
			$bindings = '@{protocol="ftp";bindingInformation="'+ $dIPV4 +':21:"}'	# Setup the Bindings for the FTP Site with the IP Address gathered above
			New-Item "IIS:\Sites\Default FTP Site" -bindings $bindings -physicalPath "%SystemDrive%\inetpub\ftproot" -Verbose:$false | Out-Null # Create site with directory
			# Ensure the settings are correct for the FTP Server to enure security
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpServer.security.ssl.controlChannelPolicy -Value 0 # Allow SSL connections
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpServer.security.ssl.dataChannelPolicy -Value 0    # Allow SSL connections
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpServer.security.authentication.basicAuthentication.enabled -Value $true # Enable Basic Authentication
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpServer.security.authentication.anonymousAuthentication.enabled -Value $false # Disable Anonymous Authentication
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpserver.userisolation.mode -Value 0  # Set USer Isolation
			Set-ItemProperty "IIS:\Sites\Default FTP Site" -Name ftpserver.logFile.directory -Value "%SystemDrive%\inetpub\logs\LogFiles"  # Set Log File Directory
			# Configure all logging options to be enabled for FTP Sites in IIS
			Set-WebConfigurationProperty -Filter "system.applicationHost/sites/siteDefaults/ftpServer/logFile" -Name LogExtFileFlags -Value "Date,Time,ClientIP,UserName,SiteName,ComputerName,ServerIP,Method,UriStem,FtpStatus,Win32Status,BytesSent,BytesRecv,TimeTaken,ServerPort,Host,FtpSubStatus,Session,FullPath,Info,ClientPort"
			# Give Authorization to SCPFtpUsers and grant "read" privileges in IIS (Default FTP Site)
			Add-WebConfiguration "/system.ftpServer/security/authorization" -value @{accessType="Allow";roles="SCPFtpUsers";permissions="Read";users=""} -PSPath IIS:\ -location "Default FTP Site"
			Restart-WebItem "IIS:\Sites\Default FTP Site"  # Restart the FTP site for all changes to take effect
		}
	}
}


####################################################################################################################################################################################
Function CreateLocalUserOrGroup($dName, $dDescription, $dType)                  # Create a Local User or Group if it does not exist
{ # Usage - CreateLocalUserOrGroup "Local [User|Group] Name" "Description" "User|Group"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dName"))) {
		invoke-command {$cn=[ADSI]"WinNT://$env:computername";$name=$cn.Create("$dType", "$dName");$name.SetInfo();$name.description=("$dDescription");$name.SetInfo()}
	}
}


####################################################################################################################################################################################
Function AddLocalUserToLocalGroup($dGroupName, $dUserName)                      # Add a Local User to a Local Group if it is not already a member
{ # Usage - AddLocalUserToLocalGroup "Local Group Name" "Local User Name"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dGroupName"))) {
		Write-Host "`t Local Group `"$env:computername\$dGroupName`" does not exist`!" -ForegroundColor Red
	}elseif (!([ADSI]::Exists("WinNT://$env:computername/$dUserName"))) {
		Write-Host "`t Local User `"$env:computername\$dUserName`" does not exist`!" -ForegroundColor Red
	}else{
		if (( @( ([ADSI]"WinNT://$env:computername/$dGroupName").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("Name", "GetProperty", $null, $_, $null);}) -notcontains $dUserName) {
			invoke-command {$group = [ADSI]"WinNT://$env:computername/$dGroupName,group";$group.Add("WinNT://$env:computername/$dUserName");$group.SetInfo()}
		}
	}
}


####################################################################################################################################################################################
Function AddDomainUserToLocalGroup($dGroupName, $dUserName)                     # Add a Domain User to a Local Group if it is not already a member
{ # Usage - AddDomainUserToLocalGroup "Local Group Name" "Domain User Name"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dGroupName"))) {
		Write-Host "`t Local Group `"$env:computername\$dGroupName`" does not exist`!" -ForegroundColor Red
	}elseif (!([ADSI]::Exists("WinNT://$env:USERDNSDOMAIN/$dUserName"))) {
		Write-Host "`t Domain User `"$env:USERDNSDOMAIN\$dUserName`" does not exist`!" -ForegroundColor Red
	}else{
		if (( @( ([ADSI]"WinNT://$env:computername/$dGroupName").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("Name", "GetProperty", $null, $_, $null);}) -notcontains $dUserName) {
			invoke-command {$group = [ADSI]"WinNT://$env:computername/$dGroupName,group";$group.Add("WinNT://$env:USERDNSDOMAIN/$dUserName");$group.SetInfo()}
		}
	}
}


####################################################################################################################################################################################
Function AddDomainUserToDomainGroup($dGroupName, $dUserName)                    # Add a Domain User to a Domain Group if it is not already a member
{ # Usage - AddDomainUserToDomainGroup "Domain Group Name" "Domain User Name"
	if (!([ADSI]::Exists("WinNT://$env:USERDNSDOMAIN/$dGroupName"))) {
		Write-Host "`t Local Group `"$env:USERDNSDOMAIN\$dGroupName`" does not exist`!" -ForegroundColor Red
	}elseif (!([ADSI]::Exists("WinNT://$env:USERDNSDOMAIN/$dUserName"))) {
		Write-Host "`t Domain User `"$env:USERDNSDOMAIN\$dUserName`" does not exist`!" -ForegroundColor Red
	}else{
		if (( @( ([ADSI]"WinNT://$env:USERDNSDOMAIN/$dGroupName").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("Name", "GetProperty", $null, $_, $null);}) -notcontains $dUserName) {
			invoke-command {$group = [ADSI]"WinNT://$env:USERDNSDOMAIN/$dGroupName,group";$group.Add("WinNT://$env:USERDNSDOMAIN/$dUserName");$group.SetInfo()}
		}
	}
}


####################################################################################################################################################################################
Function CopyGroupMembers($dSourceGroup, $dDestGroup, $dGroupType)              # Copy Group Members from one group to another
{ # Usage - CopyGroupMembers "From Group Name" "To Group Name" "Local|Domain"
	$MemberNames = @()
	if ($dGroupType -eq "local") { # If Local group is specified
		if ( ([ADSI]::Exists("WinNT://$env:computername/$dSourceGroup")) -and ([ADSI]::Exists("WinNT://$env:computername/$dDestGroup")) ) { # Check if the Local groups exist
			# Get the members for the Source Group and store them in an array
			$dMembers = ( @( ([ADSI]"WinNT://$env:computername/$dSourceGroup").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember('adspath', 'GetProperty', $null, $_, $null);} )
			foreach ($dMember in $dMembers) {
				# Add the user if it is not already a member of the destination group
				if (( @( ([ADSI]"WinNT://$env:computername/$dDestGroup,group").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("adspath", "GetProperty", $null, $_, $null);}) -notcontains "$dMember") {
					invoke-command {$group = [ADSI]"WinNT://$env:computername/$dDestGroup,group";$group.Add("$dMember");$group.SetInfo()}
				}
			}
		}
	}elseif ( ($dGroupType -eq "domain") -and ($dDomainMember) ) { # If Domain group is specified
		if ( (([ADSISearcher]"(sAMAccountName=$dSourceGroup)").FindOne()) -and (([ADSISearcher]"(sAMAccountName=$dDestGroup)").FindOne()) ) { # Check if the Domain group exists
			# Get the members for the Source Group and store them in an array
			$dMembers = ( @( ([ADSI]"WinNT://$env:USERDNSDOMAIN/$dSourceGroup").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember('adspath', 'GetProperty', $null, $_, $null);} )
			foreach ($dMember in $dMembers) {
				# Add the user if it is not already a member of the destination group
				if (( @( ([ADSI]"WinNT://$env:USERDNSDOMAIN/$dDestGroup").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("adspath", "GetProperty", $null, $_, $null);}) -notcontains "$dMember") {
					invoke-command {$group = [ADSI]"WinNT://$env:USERDNSDOMAIN/$dDestGroup,group";$group.Add("$dMember");$group.SetInfo()}
				}
			}
		}
	}
}


####################################################################################################################################################################################
Function CheckGroupMembers($dGroupName, $dUserName, $dGroupType)                # Check if a Local Users is a member of a Group (Local or Domain) - returns True or False
{ # Usage - CheckGroupMembers "Local Group Name" "Local User Name" "Local|Domain"
	$MemberNames = @()
	if ($dGroupType -eq "local") { # If Local group is specified
		if ([ADSI]::Exists("WinNT://$env:computername/$dGroupName")) { # Check if the Local group exists
			$dMembers = @( ([ADSI]"WinNT://$env:computername/$dGroupName").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("Name", "GetProperty", $null, $_, $null);}
		}
	}elseif ( ($dGroupType -eq "domain") -and ($dDomainMember) ) { # If Domain group is specified
		if (([ADSISearcher]"(sAMAccountName=$dGroupName)").FindOne()) { # Check if the Domain group exists
			$dMembers = @( ([ADSI]"WinNT://$env:USERDNSDOMAIN/$dGroupName").psbase.Invoke("Members") ) | ForEach-Object{$_.GetType().InvokeMember("Name", "GetProperty", $null, $_, $null);}
		}
	}
	($dMembers -contains "$dUserName")
}


####################################################################################################################################################################################
Function GetMembersOfGroup ($dGroupName, $dIdentifier)                          # Get all members of a group (Local or Domain) and return the required variables   
{ # Usage - GetMembersOfGroup "Administrators" "SID|Name|Caption|Domain"
	if (!($dIdentifier)) {$dIdentifier = "Name"} # If the Identifier is not specified then set it as the Name
	(Get-CimInstance -ClassName win32_group -Filter "name = '$dGroupName'" | Get-CimAssociatedInstance -Association win32_groupuser).$dIdentifier
}


####################################################################################################################################################################################
Function SetAccessToFolder ($dFolder, $dName, $dAccessRights, $dType, $dGroupType) # Set access for a User or Group to a folder if the user, group or folder exists 
{ # Usage - SetAccessToFolder "C:\HostingSpaces" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny" "Local|Domain"
	if ( ($dGroupType -eq "local") -and (!([ADSI]::Exists("WinNT://$env:computername/$dName"))) ) {
		Write-Host "`t The Local User or Group `"$env:computername\$dName`" does not exist`!" -ForegroundColor Red
	}elseif ( ($dGroupType -eq "domain") -and (!(([ADSISearcher]"(sAMAccountName=$dName)").FindOne())) ) {
		Write-Host "`t The Domain User or Group `"$env:USERDOMAIN\$dName`" does not exist`!" -ForegroundColor Red
	}elseif (!(Test-Path "$dFolder")) {
		Write-Host "`t The Folder does not exist`!" -ForegroundColor Red
	}elseif (!($dGroupType)) {
		Write-Host "`t You need to specify Local or Domain for the Group Type`!" -ForegroundColor Red
	}else{
		$AccessRule = New-Object System.Security.AccessControl.FileSystemAccessRule("$dName", "$dAccessRights", [system.security.accesscontrol.InheritanceFlags]"ContainerInherit, ObjectInherit", [system.security.accesscontrol.PropagationFlags]"None", "$dType")
		$Acl        = (Get-Item "$dFolder").GetAccessControl('Access')
		$Acl.AddAccessRule($AccessRule)
		Set-ACL -Path "$dFolder" -AclObject $Acl
		invoke-command {$AccessRule} | Out-Null
	}
}


####################################################################################################################################################################################
Function SetAccessToFolderNoChk ($dFolder, $dName, $dAccessRights, $dType)      # Set access for a User or Group to a folder if the folder exists (doesn't check the user or group)
{ # Usage - SetAccessToFolderNoChk "C:\HostingSpaces" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny"
	if (!(Test-Path "$dFolder")) {
		Write-Host "`t The Folder does not exist`!" -ForegroundColor Red
	}else{
		$AccessRule = New-Object System.Security.AccessControl.FileSystemAccessRule("$dName", "$dAccessRights", [system.security.accesscontrol.InheritanceFlags]"ContainerInherit, ObjectInherit", [system.security.accesscontrol.PropagationFlags]"None", "$dType")
		$Acl        = (Get-Item "$dFolder").GetAccessControl('Access')
		$Acl.AddAccessRule($AccessRule)
		Set-ACL -Path "$dFolder" -AclObject $Acl
		invoke-command {$AccessRule} | Out-Null
	}
}


####################################################################################################################################################################################
Function RemoveAccessToFolder ($dFolder, $dName, $dAccessRights, $dType)        # Remove access for a User or Group to a folder if the user, group or folder exists
{ # Usage - RemoveAccessToFolder "C:\HostingSpaces" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dName"))) {
		Write-Host "`t The Local User or Group `"$dName`" does not exist`!" -ForegroundColor Red
	}elseif (!(Test-Path "$dFolder")) {
		Write-Host "`t The Folder does not exist`!" -ForegroundColor Red
	}else{
		$AccessRule = New-Object System.Security.AccessControl.FileSystemAccessRule($dName,$dAccessRights, 'ContainerInherit,ObjectInherit', 'None', "$dType")
		$Acl        = (Get-Item "$dFolder").GetAccessControl('Access')
		$Acl.RemoveAccessRuleAll($AccessRule)
		Set-Acl -Path "$dFolder" -AclObject $Acl
	}
}


####################################################################################################################################################################################
Function RemoveAccessToFolderNoChk ($dFolder, $dName, $dAccessRights, $dType)   # Remove access for a User or Group to a folder if the user, group or folder exists
{ # Usage - RemoveAccessToFolderNoChk "C:\HostingSpaces" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny"
	if (!(Test-Path "$dFolder")) {
		Write-Host "`t The Folder does not exist`!" -ForegroundColor Red
	}else{
		$AccessRule = New-Object System.Security.AccessControl.FileSystemAccessRule($dName,$dAccessRights, 'ContainerInherit,ObjectInherit', 'None', "$dType")
		$Acl        = (Get-Item "$dFolder").GetAccessControl('Access')
		$Acl.RemoveAccessRuleAll($AccessRule)
		Set-Acl -Path "$dFolder" -AclObject $Acl
	}
}


####################################################################################################################################################################################
Function SetAccessToFiles ($dFileLocation, $dName, $dAccessRights, $dType)      # Set access for a User or Group to a files within a folder if the user or group exists 
{ # Usage - SetAccessToFiles "C:\HostingSpaces\*.txt" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dName"))) {
		Write-Host "`t The Local User or Group `"$dName`" does not exist`!" -ForegroundColor Red
	}else{
		Get-ChildItem $dFileLocation |  ForEach {
			$AccessRule = New-Object  System.Security.AccessControl.FileSystemAccessRule("$dName","$dAccessRights","$dType")
			$Acl        = Get-Acl "$_"
			$Acl.SetAccessRule($AccessRule)
			Set-Acl -Path "$_" -AclObject $Acl
		}
	}
}


####################################################################################################################################################################################
Function RemoveAccessToFiles ($dFileLocation, $dName, $dAccessRights, $dType)   # Grant access for a User or Group to a folder if the user or group exists 
{ # Usage - RemoveAccessToFiles "C:\HostingSpaces\*.txt" "User|Group" "Read|ReadAndExecute|Change|Modify|ListFolderContents|FullControl" "Allow|Deny"
	if (!([ADSI]::Exists("WinNT://$env:computername/$dName"))) {
		Write-Host "`t The Local User or Group `"$dName`" does not exist`!" -ForegroundColor Red
	}else{
		Get-ChildItem $dFileLocation |  ForEach {
			$AccessRule = New-Object  System.Security.AccessControl.FileSystemAccessRule($dName,"$dAccessRights","$dType")
			$Acl        = Get-Acl "$_"
			$Acl.RemoveAccessRuleAll($AccessRule)
			Set-Acl -Path "$_" -AclObject $Acl
		}
	}
}


####################################################################################################################################################################################
Function DisableFolderInheritance ($dFolder)    # Disable inheritance on the folder specified
{ # Usage - DisableFolderInheritance "C:\HostingSpaces"
if (!(Test-Path "$dFolder")) {
		Write-Host "`t The `"$dFolder`" folder does not exist`!" -ForegroundColor Red
	}else{
		$acl = Get-ACL -Path $dFolder
		$acl.SetAccessRuleProtection($True, $True)
		Set-Acl -Path $dFolder -AclObject $acl | Out-Null
	}
}


####################################################################################################################################################################################
Function EnableFolderInheritance ($dFolder)     # Enable inheritance on the folder specified
{ # Usage - EnableFolderInheritance "C:\HostingSpaces"
if (!(Test-Path "$dFolder")) {
		Write-Host "`t The `"$dFolder`" folder does not exist`!" -ForegroundColor Red
	}else{
		$acl = Get-ACL -Path $dFolder
		$acl.SetAccessRuleProtection($false, $false)
		Set-Acl -Path $dFolder -AclObject $acl | Out-Null
	}
}


####################################################################################################################################################################################
Function dCreateNewOU () # Function to create new Organisational Unit if it does not already exist
{ # Usage - dCreateNewOU - Name "OU=New OU Name" -Description "OU Description" -DiableInheritance
	Param(
		[string]$Name,              # Name for the New Organisational Unit
		[string]$Description,       # Description for the New Organisational Unit
		[switch]$DiableInheritance, # If this switch is set it will Disable Inheritance for the New Organisational Unit
		        $RemoveUserFromACL  # Remove User (Name or SID) from the ACL for the New Organisational Unit (Single User or Array of Users)
	)
	(Import-Module ActiveDirectory) | Out-Null
	$RootDN = (([ADSI]"LDAP://RootDSE").rootDomainNamingContext)
	if ( !($Name) ) { $Name = Read-Host "`n Please enter the Name for the new Organisational Unit you want to create" }
	if ( !([adsi]::Exists("LDAP://OU=$Name,$RootDN")) ) {
		$dNewOU = ([ADSI]”LDAP://$RootDN").create("organizationalUnit", "OU=$Name") ; $dNewOU.setInfo()
		# Set the Description for the new OU if it has been specified
		if ($Description) { $dNewOU.description=("$Description");$dNewOU.SetInfo() }
		# Check if the New OU has been created before continuing
		do { $dCeckIfOUcreated = (Test-Path "AD:\OU=$Name,$RootDN") } until ($dCeckIfOUcreated -eq $true)
		Write-Host "`t Created a New Organisational Unit Called `"$Name`"" -ForegroundColor Green
		# Disable Inheritance if requested
		if ($DiableInheritance) {
			$acl1 = Get-ACL -Path "AD:\OU=$Name,$RootDN"
			$acl1.SetAccessRuleProtection($True, $True)
			Set-Acl -Path "AD:\OU=$Name,$RootDN" -AclObject $acl1 | Out-Null
			Write-Host "`t Disabling Inheritance on the `"$Name`" Organisational Unit" -ForegroundColor Green
		}
		# Remove access 
		if ($RemoveUserFromACL) {
			foreach ($user in $RemoveUserFromACL) {
				$acl = Get-Acl -Path "AD:\OU=$Name,$RootDN"
				foreach($acc in $acl.access ) {
					if(($acc.IdentityReference.Value) -eq "$user") {
						($acl.RemoveAccessRule($acc)) | Out-Null
						(Set-Acl -Path "AD:\OU=$Name,$RootDN" -AclObject $acl -ErrorAction Stop) | Out-Null
					}
				}
				Write-Host "`t Removing access for `"$user`" on the `"$Name`" Organisational Unit" -ForegroundColor Green
			}
		}
	}
}


####################################################################################################################################################################################
Function InstallDisablePasswdComplex()  # Disable Password Complexity Policy – Local Security Policy
{
	Write-Host "`tDisabling Password Complexity Policy – Local Security Policy" -ForegroundColor Cyan
	(secedit /export /cfg c:\secpol.cfg) | Out-Null;
	((gc C:\secpol.cfg).replace("PasswordComplexity = 1", "PasswordComplexity = 0") | Out-File C:\secpol.cfg) | Out-Null;
	(secedit /configure /db c:\windows\security\local.sdb /cfg c:\secpol.cfg /areas SECURITYPOLICY) | Out-Null;
	rm -force c:\secpol.cfg -confirm:$false
}


####################################################################################################################################################################################
Function InstallIISforESandPortal()     # Function to install additional IIS features required by the SolidCP Enterprise Server and Portal
{
	Write-Host "`tInstalling the features required for the SolidCP Enterprise Server and Portal" -ForegroundColor Cyan
	# Install the additional features required for SolidCP to be able to send emails from the Enterpeise Server
	(Add-WindowsFeature -Name Web-Lgcy-Mgmt-Console, SMTP-Server) | Out-Null
	# Change the "Default Website" port from 80 to 8080 so that we can install SolidCP on Port 80
	if ( (Get-WebBinding -Name 'Default Web Site').bindingInformation -eq "*:80:" ) {
		Set-WebBinding -Name 'Default Web Site' -BindingInformation "*:80:" -PropertyName Port -Value 8080
	}
}


####################################################################################################################################################################################
Function InstallDNSServer()         # Function to install features for Microsoft DNS Server
{
	Write-Host "`tInstalling the features required for Microsoft DNS Server" -ForegroundColor Cyan
	# Install the additional features required for the Microsoft DNS Server
	(Add-WindowsFeature -Name DNS, RSAT-DNS-Server) | Out-Null
	# Remove the DNS Root Hints for security and to stop the DNS Servers being used in Amplification Attacks
	Get-DnsServerRootHint | Remove-DnsServerRootHint -force
}


####################################################################################################################################################################################
Function InstallRDShostServer()     # Function to install features for the RDS Host Server
{
	Write-Host "`tInstalling the features required for the Remote Desktop Host Server" -ForegroundColor Cyan
	# Install the additional features required for the Microsoft RDS Host Server (RemoteApps or Remote Desktop)
	(Add-WindowsFeature -Name RSAT-AD-PowerShell) | Out-Null
}


####################################################################################################################################################################################
Function InstallActiveDirectory()   # Function to install features for Active Directory
{
	# ********** Domain Member, but signed in locally **********
	if ( ($dDomainMember) -and ($dLoggedInLocally) ) {
		Write-Host "`n`t *************************************************" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *      This server is a member of a domain      *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *    You need to login with a domain account    *" -ForegroundColor Yellow
		Write-Host "`t *          to promote this server to a          *" -ForegroundColor Yellow
		Write-Host "`t *               Domain Controller               *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *  Please log back on with an account that is   *" -ForegroundColor Yellow
		Write-Host "`t *      a member of the Domain Admins group      *" -ForegroundColor Yellow
		Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *************************************************" -ForegroundColor Yellow
	}else{
		Write-Host "`tInstalling the features required for Active Directory" -ForegroundColor Cyan
		# Install the features required for this server to run Active Directory (Domain Controllers)
		(Add-WindowsFeature -Name AD-Domain-Services, RSAT-AD-Tools, RSAT-AD-PowerShell, RSAT-ADDS, RSAT-AD-AdminCenter, RSAT-ADDS-Tools, WDS-AdminPack -IncludeManagementTools) | Out-Null

		# Ask the user if they want to configure Active Directory on this server
		$InstallAD = ""; while ($InstallAD -notmatch "[y|n]") { $InstallAD = read-host "`t Do you want to configure Active Directory on this server`? (Y/N)" }
		if ($InstallAD -eq "n") { # If the user answers No to the question, let them know they must manually promote this server to a Domain Controller
			Write-Host "`n`t *************************************************" -ForegroundColor Yellow
			Write-Host "`t *                                               *" -ForegroundColor Yellow
			Write-Host "`t *   The Windows Features for Active Directory   *" -ForegroundColor Yellow
			Write-Host "`t *       have been successfully installed        *" -ForegroundColor Yellow
			Write-Host "`t *                                               *" -ForegroundColor Yellow
			Write-Host "`t *    You will need to manually promote this     *" -ForegroundColor Yellow
			Write-Host "`t *         server to a Domain Controller         *" -ForegroundColor Yellow
			Write-Host "`t *                                               *" -ForegroundColor Yellow
			Write-Host "`t *         We reccomend having at least          *" -ForegroundColor Yellow
			Write-Host "`t *      2 Domain Controllers for redundancy      *" -ForegroundColor Yellow
			Write-Host "`t *                                               *" -ForegroundColor Yellow
			Write-Host "`t *************************************************" -ForegroundColor Yellow
		}elseif ($InstallAD -eq "y") { # Configure Active Directory if the user answers Yes to the question
			# ********** First Domain Controller **********
			if (!$dDomainMember) {
				# Check with the user if they want to configure this server as the First Domain Controller
				$InstallADprimary = ""; while ($InstallADprimary -notmatch "[y|n]") { 
				Write-Host "`t This server is NOT a member of a domain!" -ForegroundColor Yellow
				$InstallADprimary = read-host "`t Do you want to configure this server as a Primary Domain Controller`? (Y/N)"
			}
				if ($InstallADprimary -eq "y") { # Configure this server as the Primary Domain Controller if the user answers Yes to the question
				# If a Domain Name has not been defined in the user variables ask the user to enter one
				if (!$dSCPdomainName) { $dSCPdomainName = Read-Host "Please enter the Domain Name you would like to use for Active Directory" }
				Write-Host "`t Configuring this server as the first Domain Controller`n" -ForegroundColor Green
				# Import the Active Directory Deployment Module
				Import-Module ADDSDeployment
				# Configure this server as the First Domain Controller in a new forest
				(Install-ADDSForest -CreateDnsDelegation:$false -DomainName "$dSCPdomainName" -InstallDns:$true -NoRebootOnCompletion:$true -Force:$true -WarningAction SilentlyContinue) | Out-Null
				Write-Host "`n`t *************************************************" -ForegroundColor Yellow
				Write-Host "`t *                                               *" -ForegroundColor Yellow
				Write-Host "`t *       This server has been successfully       *" -ForegroundColor Yellow
				Write-Host "`t *        promoted to a Domain Controller        *" -ForegroundColor Yellow
				Write-Host "`t *                                               *" -ForegroundColor Yellow
				Write-Host "`t *         We reccomend having at least          *" -ForegroundColor Yellow
				Write-Host "`t *      2 Domain Controllers for redundancy      *" -ForegroundColor Yellow
				Write-Host "`t *                                               *" -ForegroundColor Yellow
				Write-Host "`t *                                               *" -ForegroundColor Yellow
				Write-Host "`t *        The server will now be rebooted        *" -ForegroundColor Yellow
				Write-Host "`t *                                               *" -ForegroundColor Yellow
				Write-Host "`t *************************************************" -ForegroundColor Yellow
				Write-Host "`n`t Press any key to reboot this server" -ForegroundColor Green
				$dRestartConfirm = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
				if ($dRestartConfirm) {Restart-Computer}
			}
			# ********** Additional Domain Controller **********
			}elseif ( ($dDomainMember) -and (!$dLoggedInLocally) ) {
				# Check with the user if they want to configure this server as an Additional Domain Controller
				$InstallADsecondary = ""; while ($InstallADsecondary -notmatch "[y|n]") {
					Write-Host "`t This server is a member of a domain!" -ForegroundColor Yellow
					$InstallADsecondary = read-host "`t Do you want to configure this server as an Additional Domain Controller`? (Y/N)"
				}
				if ($InstallADsecondary -eq "y") { # Configure this server as an Additional Domain Controller if the user answers Yes to the question
					Write-Host "`t Configuring this server as an additional Domain Controller`n" -ForegroundColor Green
					# Import the Active Directory Deployment Module
					Import-Module ADDSDeployment
					# Configure this server as an additional Domain Controller in an existing forest
					if (CheckGroupMembers "$dLangDomainAdminsGroup" "$dLoggedInUserName" "Domain") {
						(Install-ADDSDomainController -CreateDnsDelegation:$false  -DomainName "$env:USERDNSDOMAIN" -InstallDns:$true -NoRebootOnCompletion:$true -Force:$true -WarningAction SilentlyContinue) | Out-Null
					}else{
						(Install-ADDSDomainController -CreateDnsDelegation:$false  -DomainName "$env:USERDNSDOMAIN" -InstallDns:$true -NoRebootOnCompletion:$true -Force:$true -Credential (Get-Credential "$env:USERDOMAIN\$dLangDomainAdministratorName") -WarningAction SilentlyContinue) | Out-Null
					}
					Write-Host "`n`t *************************************************" -ForegroundColor Yellow
					Write-Host "`t *                                               *" -ForegroundColor Yellow
					Write-Host "`t *       This server has been successfully       *" -ForegroundColor Yellow
					Write-Host "`t *  promoted to an additional Domain Controller  *" -ForegroundColor Yellow
					Write-Host "`t *                                               *" -ForegroundColor Yellow
					Write-Host "`t *                                               *" -ForegroundColor Yellow
					Write-Host "`t *        The server will now be rebooted        *" -ForegroundColor Yellow
					Write-Host "`t *                                               *" -ForegroundColor Yellow
					Write-Host "`t *************************************************" -ForegroundColor Yellow
					Write-Host "`n`t Press any key to reboot this server" -ForegroundColor Green
					$dRestartConfirm = $host.ui.RawUI.ReadKey("NoEcho,IncludeKeyDown")
					if ($dRestartConfirm) {Restart-Computer}
				}
			}
		}
	}
}


####################################################################################################################################################################################
Function InstallActivePerl()            # Function to install Perl - ActiverPerl v5.22.1
{
	# Check if Perl is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Perl64") -or (Test-Path "C:\Perl\") ) {
		# Perl is not installed on this server so go ahead and install it
		Write-Host "`tDetermining your Operating System Type and installing the correct version of Perl" -ForegroundColor Cyan
		# Create the Perl ActivePerl Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\Perl - ActivePerl v5.22.1' -Force) | Out-Null ; cd 'C:\_Install Files\Perl - ActivePerl v5.22.1\'
		# Download the files required for Perl, then install them. This downloads the correct version for the OS and then installs it.
		if ([Environment]::Is64BitProcess) {
			Write-Host "`t Downloading the 64bit version of Perl" -ForegroundColor Green
			(New-Object System.Net.WebClient).DownloadFile("http://downloads.activestate.com/ActivePerl/releases/5.22.1.2201/ActivePerl-5.22.1.2201-MSWin32-x64-299574.msi", "C:\_Install Files\Perl - ActivePerl v5.22.1\ActivePerl-5.22.1.2201-MSWin32-x64-299574.msi") | Out-Null
			Write-Host "`t Installing the 64bit version of Perl" -ForegroundColor Green
			(Start-Process -FilePath 'C:\_Install Files\Perl - ActivePerl v5.22.1\ActivePerl-5.22.1.2201-MSWin32-x64-299574.msi' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
			# Add the Perl ISAPI and CGI Restrictions for Perl (64 bit)
			Write-Host "`t Adding the Perl ISAPI and CGI Restrictions in IIS" -ForegroundColor Green
			Add-WebConfiguration -pspath 'MACHINE/WEBROOT/APPHOST' -filter 'system.webServer/security/isapiCgiRestriction' -value @{description='Perl FastCGI' ; path= "C:\Perl64\bin\perl.exe `"%s`" %s" ; allowed='True'}
			# Add the Perl Handler Mappings for Perl (64 bit)
			Write-Host "`t Adding the Perl Handler Mappings in IIS" -ForegroundColor Green
			New-WebHandler -Name "Perl_Script_Map_PL" -Path "*.pl" -Verb '*' -Modules CgiModule -ScriptProcessor 'C:\Perl64\bin\perl.exe "%s" %s' -ResourceType File
			New-WebHandler -Name "Perl_Script_Map_CGI" -Path "*.cgi" -Verb '*' -Modules CgiModule -ScriptProcessor 'C:\Perl64\bin\perl.exe "%s" %s' -ResourceType File
		}else{
			Write-Host "`t Downloading the 32bit version of Perl" -ForegroundColor Green
			(New-Object System.Net.WebClient).DownloadFile("http://downloads.activestate.com/ActivePerl/releases/5.22.1.2201/ActivePerl-5.22.1.2201-MSWin32-x86-64int-299574.msi", "C:\_Install Files\Perl - ActivePerl v5.22.1\ActivePerl-5.22.1.2201-MSWin32-x86-64int-299574.msi") | Out-Null
			Write-Host "`t Installing the 32bit version of Perl" -ForegroundColor Green
			(Start-Process -FilePath 'C:\_Install Files\Perl - ActivePerl v5.22.1\ActivePerl-5.22.1.2201-MSWin32-x86-64int-299574.msi' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
			# Add the Perl ISAPI and CGI Restrictions for Perl (32 bit)
			Write-Host "`t Adding the Perl ISAPI and CGI Restrictions in IIS" -ForegroundColor Green
			Add-WebConfiguration -pspath 'MACHINE/WEBROOT/APPHOST' -filter 'system.webServer/security/isapiCgiRestriction' -value @{description='Perl FastCGI' ; path= "C:\Perl\bin\perl.exe `"%s`" %s" ; allowed='True'}
			# Add the Perl Handler Mappings for Perl (32 bit)
			Write-Host "`t Adding the Perl Handler Mappings in IIS" -ForegroundColor Green
			New-WebHandler -Name "Perl_Script_Map_PL" -Path "*.pl" -Verb '*' -Modules CgiModule -ScriptProcessor 'C:\Perl\bin\perl.exe "%s" %s' -ResourceType File
			New-WebHandler -Name "Perl_Script_Map_CGI" -Path "*.cgi" -Verb '*' -Modules CgiModule -ScriptProcessor 'C:\Perl\bin\perl.exe "%s" %s' -ResourceType File
		}
	cd '\'
	}
}


####################################################################################################################################################################################
Function InstallWebPI()                                 # Function to install Microsoft Web Platform Installer
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) {
		# WebPI is not installed on this server so go ahead and install it
		Write-Host "`tDetermining your Operating System Type and installing the correct version of WebPI" -ForegroundColor Cyan
		# Create the WebPI Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\WebPI' -Force) | Out-Null ; cd 'C:\_Install Files\WebPI\'
		if ([Environment]::Is64BitProcess){ # Download and install the 64 bit version if the operating system is 64 bit.
			Write-Host "`t Downloading the 64bit version of Web Platform Installer" -ForegroundColor Green
			(New-Object System.Net.WebClient).DownloadFile("http://download.microsoft.com/download/C/F/F/CFF3A0B8-99D4-41A2-AE1A-496C08BEB904/WebPlatformInstaller_amd64_en-US.msi", "C:\_Install Files\WebPI\WebPlatformInstaller_amd64_en-US.msi") | Out-Null
			Write-Host "`t installing the 64bit version of Web Platform Installer" -ForegroundColor Green
			(Start-Process -FilePath 'C:\_Install Files\WebPI\WebPlatformInstaller_amd64_en-US.msi' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
		}else{								# Download and install the 32 bit version if the operating system is 32 bit.
			Write-Host "`t Downloading the 32bit version of Web Platform Installer" -ForegroundColor Green
			(New-Object System.Net.WebClient).DownloadFile("http://download.microsoft.com/download/C/F/F/CFF3A0B8-99D4-41A2-AE1A-496C08BEB904/WebPlatformInstaller_x86_en-US.msi", "C:\_Install Files\WebPI\WebPlatformInstaller_x86_en-US.msi") | Out-Null
			Write-Host "`t installing the 32bit version of Web Platform Installer" -ForegroundColor Green
			(Start-Process -FilePath 'C:\_Install Files\WebPI\WebPlatformInstaller_x86_en-US.msi' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
		}
		cd 'C:\'
		start-Sleep -Seconds 10
	}
}


####################################################################################################################################################################################
Function InstallWebPI_PHP()                             # Function to install PHP from the WebPI Command Line
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) { InstallWebPI }
	# Install PHP and PHP Manager for IIS if it is not already installed
	Write-Host "`tWebPI is installing PHP (v5.3, v5.4, v5.5, v5.6) and PHP Manager for IIS" -ForegroundColor Cyan
	cd "C:\Program Files\Microsoft\Web Platform Installer\"
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.3\") ) { Write-Host "`t Installing PHP v5.3" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:PHP53 /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.4\") ) { Write-Host "`t Installing PHP v5.4" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:PHP54 /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.5\") ) { Write-Host "`t Installing PHP v5.5" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:PHP55 /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.6\") ) { Write-Host "`t Installing PHP v5.6" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:PHP56 /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files\PHP Manager 1.2 for IIS 7\") ) { Write-Host "`t Installing PHP Manager for IIS" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:PHPManager /AcceptEULA}) | Out-Null }
	cd '\'
}


####################################################################################################################################################################################
Function InstallWebPI_Python()                          # Function to install Python v3.4 from the WebPI Command Line
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) { InstallWebPI }
	# Install Python v3.4 if it is not already installed
	Write-Host "`tWebPI is installing Python v3.4 for IIS" -ForegroundColor Cyan
	cd "C:\Program Files\Microsoft\Web Platform Installer\"
	if ( !(Test-Path "C:\Python34\") ) { Write-Host "`t Installing Python v3.4" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:Python34 /AcceptEULA}) | Out-Null }
	cd '\'
}


####################################################################################################################################################################################
Function InstallWebPI_SQLdriverPHP()                    # Function to install the SQL Driver for PHP from the WebPI Command Line
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) { InstallWebPI }
	# Install SQL Drivers for PHP if they are not already installed
	Write-Host "`tWebPI is installing the SQL Driver for PHP" -ForegroundColor Cyan
	cd "C:\Program Files\Microsoft\Web Platform Installer\"
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.3\ext\php_sqlsrv.dll") ) { Write-Host "`t Installing the SQL Driver for PHP v5.3" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:SQLDriverPHP53IIS /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.4\ext\php_sqlsrv.dll") ) { Write-Host "`t Installing the SQL Driver for PHP v5.4" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:SQLDriverPHP54IIS /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.5\ext\php_sqlsrv.dll") ) { Write-Host "`t Installing the SQL Driver for PHP v5.5" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:SQLDriverPHP55IIS /AcceptEULA}) | Out-Null }
	if ( !(Test-Path "C:\Program Files (x86)\PHP\v5.6\ext\php_sqlsrv.dll") ) { Write-Host "`t Installing the SQL Driver for PHP v5.6" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:SQLDriverPHP56IIS /AcceptEULA}) | Out-Null }
	cd '\'
}


####################################################################################################################################################################################
Function InstallWebPI_URLreWrite()                      # Function to install the URL Rewrite module from the WebPI Command Line
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) { InstallWebPI }
	# Install URL ReWrite for IIS if it is not already installed
	Write-Host "`tWebPI is installing the URL ReWrite module for IIS" -ForegroundColor Cyan
	cd "C:\Program Files\Microsoft\Web Platform Installer\"
	if ( !(Test-Path "HKLM:\SOFTWARE\Microsoft\IIS Extensions\URL Rewrite\") ) { Write-Host "`t Installing the URL ReWrite module for IIS" ; (invoke-command {.\WebPICMD.exe /Install /Products:UrlRewrite2 /AcceptEULA}) | Out-Null }
	cd '\'
}


####################################################################################################################################################################################
Function InstallWebPI_WebDeploy()                       # Function to install the Web Deploy module from the WebPI Command Line
{
	# Check if WebPI is installed, if not then install the correct version for the server
	if ( !(Test-Path "C:\Program Files\Microsoft\Web Platform Installer\") ) { InstallWebPI }
	# Install WebDeploy if it is not already installed
	Write-Host "`tWebPI is installing the Web Deploy module for IIS" -ForegroundColor Cyan
	cd "C:\Program Files\Microsoft\Web Platform Installer\"
	if ( !(Test-Path "C:\Program Files\IIS\Microsoft Web Deploy V3\") ) { Write-Host "`t Installing the Web Deploy module for IIS" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:WDeployNoSMO /AcceptEULA}) | Out-Null }
	cd '\'
}


####################################################################################################################################################################################
Function InstallMariaDB_MySQL($dMariaMySQLpasswd)       # Function to install the MariaDB or MySQL Database Server
{
	# Run the function to check which Database Server (MariaDB or MySQL) needs to be installed and request a password unless it has been defined
	dCheckMariaMySQLpassword

	if ($dMySQLvMariaDB -eq "MariaDB") { # Install MariaDB if required
		if ( !(Test-Path "C:\Program Files\MariaDB 10.1\") ) { # Check if the MariaDB Database Server is already installed
			Write-Host "`tDownloading the MariaDB Database Server" -ForegroundColor Cyan
			# Create the MariaDB Database Server Directory in our Installation Files folder ready for downloading
			(md -Path 'C:\_Install Files\MariaDB Database Server' -Force) | Out-Null ; cd 'C:\_Install Files\MariaDB Database Server\'
			# Download MariaDB Database Server
			(New-Object System.Net.WebClient).DownloadFile("http://ftp.hosteurope.de/mirror/archive.mariadb.org/mariadb-10.1.14/winx64-packages/mariadb-10.1.14-winx64.msi", "C:\_Install Files\MariaDB Database Server\mariadb-10.1.14-winx64.msi")
			# Install the MariaDB Database Server x64 on the Server
			if ( !(Test-Path "C:\Program Files\MariaDB 10.1\") ) { # Check if the MariaDB Database Server is already installed
				Write-Host "`t Installing MariaDB Database Server`n" -ForegroundColor Green
				((Start-Process -FilePath 'C:\_Install Files\MariaDB Database Server\mariadb-10.1.14-winx64.msi' -Argumentlist "/QN PASSWORD=$dMariaMySQLpasswd ALLOWREMOTEROOTACCESS=1 UTF8=1 SERVICENAME=MariaDB /passive" -Wait -Passthru).ExitCode) | Out-Null
			}
		}
	}elseif ($dMySQLvMariaDB -eq "MySQL") { # Install MySQL if required
		Write-Host "`tInstalling the MySQL Database Server" -ForegroundColor Cyan
		cd "C:\Program Files\Microsoft\Web Platform Installer\"
		if ( !(Test-Path "C:\Program Files\MySQL\MySQL Server 5.1\") ) { # Check if the MySQL Database Server is already installed
			# Install the MySQL Database Server
			Write-Host "`t Installing the MySQL Database Server" -ForegroundColor Green
			(invoke-command {.\WebPICMD.exe /Install /Products:MySQL /AcceptEULA /MySQLPassword:$dMariaMySQLpasswd}) | Out-Null
		}
		# Add the firewall rule to open Port 3306 for inbound management of the MySQL Server 5.1 only if it doesn't exist
		if ( !(Get-NetFirewallRule | where DisplayName -EQ "MySQL Server - Port 3306") ) {
			Write-Host "`tOpening port 3306 for the MySQL Database Server on the Firewall" -ForegroundColor Cyan
			(New-NetFirewallRule -DisplayName "MySQL Server - Port 3306" -Direction Inbound –LocalPort 3306 -Protocol TCP -Action Allow) | Out-Null
		}
		cd '\'
	}
	# Check if the MySQL Connector Net v6.9.7 is already installed (also works for MarioaDB), if not then install it via the Web Platform Installer
	if ( !(Test-Path "C:\Program Files (x86)\MySQL\MySQL Connector Net 6.9.7\") ) {
		# Install the MySQL Connector Net v6.9.7
		Write-Host "`t Installing the MySQL Connector Net v6.9.7" -ForegroundColor Green ; (invoke-command {.\WebPICMD.exe /Install /Products:MySQLConnector /AcceptEULA}) | Out-Null
	}
}


####################################################################################################################################################################################
Function InstallSQLsvr2014withTools($dMsSQLpassword)    # Function to install SQL Server 2014 Express (with tools)
{
	if ( !(Test-Path "C:\Program Files\Microsoft SQL Server\MSSQL12.SQLEXPRESS\MSSQL\DATA\") ) {
		# Run the function to request a password for the Microsoft SQL Server unless it has been defined
		dCheckMsSQLpassword

		Write-Host "`tDownloading SQL Server 2014 Express (with tools)" -ForegroundColor Cyan
		# Create the SQL Server 2014 Express with Tools Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\SQL Server 2014 Express with Tools' -Force) | Out-Null ; cd 'C:\_Install Files\SQL Server 2014 Express with Tools\'
		# Download SQL Server 2014 Express with Tools x64 from Microsoft
		(New-Object System.Net.WebClient).DownloadFile("http://download.microsoft.com/download/E/A/E/EAE6F7FC-767A-4038-A954-49B8B05D04EB/ExpressAndTools%2064BIT/SQLEXPRWT_x64_ENU.exe", "C:\_Install Files\SQL Server 2014 Express with Tools\SQLEXPRWT_x64_ENU.exe")

		# Install the SQL Server 2014 Express with Tools x64 on the Server
		Write-Host "`t Extracting and Installing SQL Server 2014 Express (with tools)`n" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *     This will take quite a while     *" -ForegroundColor Green
		Write-Host "`t    *           to fully complete          *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *  Please be patient while we install  *" -ForegroundColor Green
		Write-Host "`t    *       SQL Server 2014 Express        *" -ForegroundColor Green
		Write-Host "`t    *        With Management Tools         *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		((Start-Process -FilePath 'C:\_Install Files\SQL Server 2014 Express with Tools\SQLEXPRWT_x64_ENU.exe' -Argumentlist "/Q /IACCEPTSQLSERVERLICENSETERMS=1 /ADDCURRENTUSERASSQLADMIN=True /ACTION=Install /INSTANCEID=SQLEXPRESS /INSTANCENAME=SQLExpress /TCPENABLED=1 /UPDATEENABLED=True /FEATURES=SQLENGINE,REPLICATION,CONN,BC,SDK,SSMS,ADV_SSMS,SNAC_SDK,Tools /SECURITYMODE=SQL /SAPWD=$dMsSQLpassword /INDICATEPROGRESS=False /SQMREPORTING=False /AGTSVCSTARTUPTYPE=Automatic /SQLSVCSTARTUPTYPE=Automatic /SQLCOLLATION=`"SQL_Latin1_General_CP1_CI_AS`" /BROWSERSVCSTARTUPTYPE=Automatic" -Wait -Passthru).ExitCode) | Out-Null

		InstallSQLsvr2012NativeClnt
		InstallMSSQL_TCPports
		InstallFirewall_MSSQL
	}
}


####################################################################################################################################################################################
Function InstallSQLsvr2014noTools($dMsSQLpassword2)     # Function to install SQL Server 2014 Express ONLY (NO Tools)
{
	if ( !(Test-Path "C:\Program Files\Microsoft SQL Server\MSSQL12.SQLEXPRESS\MSSQL\DATA\") ) {
		# Run the function to request a password for the Microsoft SQL Server unless it has been defined
		dCheckMsSQLpassword

		Write-Host "`tDownloading SQL Server 2014 Express" -ForegroundColor Cyan
		# Create the SQL Server 2014 Express with Tools Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\SQL Server 2014 Express' -Force) | Out-Null ; cd 'C:\_Install Files\SQL Server 2014 Express\'
		# Download SQL Server 2014 Express x64 from Microsoft
		(New-Object System.Net.WebClient).DownloadFile("https://download.microsoft.com/download/E/A/E/EAE6F7FC-767A-4038-A954-49B8B05D04EB/Express%2064BIT/SQLEXPR_x64_ENU.exe", "C:\_Install Files\SQL Server 2014 Express\SQLEXPR_x64_ENU.exe")

		# Install the SQL Server 2014 Express x64 on the Server
		Write-Host "`t Extracting and Installing SQL Server 2014 Express`n" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *     This will take quite a while     *" -ForegroundColor Green
		Write-Host "`t    *           to fully complete          *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *  Please be patient while we install  *" -ForegroundColor Green
		Write-Host "`t    *       SQL Server 2014 Express        *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		((Start-Process -FilePath 'C:\_Install Files\SQL Server 2014 Express\SQLEXPR_x64_ENU.exe' -Argumentlist "/Q /IACCEPTSQLSERVERLICENSETERMS=1 /ADDCURRENTUSERASSQLADMIN=True /ACTION=Install /INSTANCEID=SQLEXPRESS /INSTANCENAME=SQLExpress /TCPENABLED=1 /UPDATEENABLED=True /FEATURES=SQLENGINE,REPLICATION,CONN,BC,SDK,SNAC_SDK /SECURITYMODE=SQL /SAPWD=$dMsSQLpassword2 /INDICATEPROGRESS=False /SQMREPORTING=False /AGTSVCSTARTUPTYPE=Automatic /SQLSVCSTARTUPTYPE=Automatic /SQLCOLLATION=`"SQL_Latin1_General_CP1_CI_AS`" /BROWSERSVCSTARTUPTYPE=Automatic" -Wait -Passthru).ExitCode) | Out-Null

		InstallSQLsvr2012NativeClnt
		InstallMSSQL_TCPports
		InstallFirewall_MSSQL
	}
}


####################################################################################################################################################################################
Function InstallSQLsvr2014mgmtTools()                   # Function to install SQL Server 2014 Express Management Tools
{
	if ( !(Test-Path "C:\Program Files (x86)\Microsoft SQL Server\120\Tools\Binn\ManagementStudio\Ssms.exe") ) { 
		Write-Host "`tDownloading SQL Server 2014 Express Management Tools" -ForegroundColor Cyan
		# Create the SQL Server 2014 Express with Tools Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\SQL Server 2014 Express Management Tools' -Force) | Out-Null ; cd 'C:\_Install Files\SQL Server 2014 Express Management Tools\'
		# Download SQL Server 2014 Express Management Tools x64 from Microsoft
		(New-Object System.Net.WebClient).DownloadFile("https://download.microsoft.com/download/E/A/E/EAE6F7FC-767A-4038-A954-49B8B05D04EB/MgmtStudio%2064BIT/SQLManagementStudio_x64_ENU.exe", "C:\_Install Files\SQL Server 2014 Express Management Tools\SQLManagementStudio_x64_ENU.exe")

		# Install the SQL Server 2014 Express with Tools x64 on the Server
		Write-Host "`t Extracting and Installing SQL Server 2014 Express Management Tools`n" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *     This will take quite a while     *" -ForegroundColor Green
		Write-Host "`t    *           to fully complete          *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *  Please be patient while we install  *" -ForegroundColor Green
		Write-Host "`t    *       SQL Server 2014 Express        *" -ForegroundColor Green
		Write-Host "`t    *        Management Tools ONLY         *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		((Start-Process -FilePath 'C:\_Install Files\SQL Server 2014 Express Management Tools\SQLManagementStudio_x64_ENU.exe' -Argumentlist "/Q /IACCEPTSQLSERVERLICENSETERMS=1 /ACTION=Install /UPDATEENABLED=True /FEATURES=SSMS,ADV_SSMS,Tools /INDICATEPROGRESS=False" -Wait -Passthru).ExitCode) | Out-Null
	}
}


####################################################################################################################################################################################
Function InstallSQLsvr2012NativeClnt()                  # Function to install SQL Server 2012 Native Client
{
	if ( !(Test-Path "C:\Program Files\Microsoft SQL Server\110\SDK\Lib\x64\sqlncli11.lib") ) { 
		# Stopping the SQL Server Service
		Write-Host "`tStopping the SQL Server ready for the SQL Server 2012 Native Client Installation" -ForegroundColor Cyan
		(Stop-Service 'MSSQL$SQLExpress' -Force -WarningAction SilentlyContinue) | Out-Null
		Write-Host "`t Downloading the SQL Server 2012 Native Client" -ForegroundColor Green
		# Create the SQL Server 2012 Native Client Directory in our Installation Files folder ready for downloading
		(md -Path 'C:\_Install Files\SQL Server 2012 Native Client' -Force) | Out-Null ; cd 'C:\_Install Files\SQL Server 2012 Native Client\'
		# Download the SQL Server Native Client from Microsoft
		(New-Object System.Net.WebClient).DownloadFile("http://download.microsoft.com/download/4/B/1/4B1E9B0E-A4F3-4715-B417-31C82302A70A/ENU/x64/sqlncli.msi", "C:\_Install Files\SQL Server 2012 Native Client\sqlncli.msi")
		# Install the SQL Server Native Client on the Server
		Write-Host "`t Installing the SQL Server 2012 Native Client" -ForegroundColor Green
		((Start-Process -FilePath "C:\_Install Files\SQL Server 2012 Native Client\sqlncli.msi" -ArgumentList "/qb IACCEPTSQLNCLILICENSETERMS=YES" -Wait -Passthru).ExitCode) | Out-Null
		# Start the SQL Server Service
		Write-Host "`t Starting the SQL Server again as we have just installed the SQL Server 2012 Native Client" -ForegroundColor Green
		(Start-Service 'MSSQL$SQLExpress' -WarningAction SilentlyContinue) | Out-Null
	}
}


####################################################################################################################################################################################
Function InstallMSSQL_TCPports()                        # Function to the TCP Ports and Named Pipes for the Microsoft SQL Database Server
{
	$dEnableSQLports = @'
	# Setup the enviroment to enable the ports
	$wmi = New-Object ('Microsoft.SqlServer.Management.Smo.Wmi.ManagedComputer')
	$Tcp = $wmi.GetSmoObject("ManagedComputer[@Name='$env:COMPUTERNAME']/ ServerInstance[@Name='SQLExpress']/ServerProtocol[@Name='Tcp']")

	# Enable TCP on the SQL Server
	$Tcp.IsEnabled = $true
	$TCP.alter()

	# Enabled TCP, Disbale Dynamic Ports and Set TCP Port as 1433 for all IP Addresses
	foreach ($ipAddress in $Tcp.IPAddresses)
	{
		if ($ipAddress.IPAddressProperties["Active"].Value) {
			$ipAddress.IPAddressProperties["Enabled"].Value=$True
		}
		$ipAddress.IPAddressProperties["TcpDynamicPorts"].Value = ""
		$ipAddress.IPAddressProperties["TcpPort"].Value = "1433"
	}
	$Tcp.Alter()

	# Enable the named pipes protocol for the default instance.
	$NamedPipes = $wmi.GetSmoObject("ManagedComputer[@Name='$env:COMPUTERNAME']/ ServerInstance[@Name='SQLExpress']/ServerProtocol[@Name='Np']")
	$NamedPipes.IsEnabled = $true
	$NamedPipes.Alter()

	# Exit the SQL PowerShell Enviroment
	exit
'@

	Write-Host "`tEnabling TCP and Named Pipes for the Microsoft SQL Database Server" -ForegroundColor Cyan
	# Register the SQL PowerShell Modules
	start-Sleep -Seconds 25
	(set-alias installutil $env:windir\microsoft.net\framework\v2.0.50727\installutil) | Out-Null
	(installutil -i “C:\Program Files (x86)\Microsoft SQL Server\120\Tools\PowerShell\Modules\SQLPS\Microsoft.SqlServer.Management.PSProvider.dll”) | Out-Null
	(installutil -i “C:\Program Files (x86)\Microsoft SQL Server\120\Tools\PowerShell\Modules\SQLPS\Microsoft.SqlServer.Management.PSSnapins.dll”) | Out-Null
	# Open the SQL PowerShell Enviroment and run the above command
	cd 'C:\Program Files (x86)\Microsoft SQL Server\120\Tools\Binn'
	invoke-expression "cmd /c start sqlps -Command {$dEnableSQLports}"
	# Restart the SQL Server Service
	Write-Host "`t Restarting the SQL Database Server" -ForegroundColor Yellow
	(Restart-Service 'MSSQL$SQLExpress' -Force -WarningAction SilentlyContinue) | Out-Null
	Write-Host "`t Restarted successfully" -ForegroundColor Green
	sleep 1
}


####################################################################################################################################################################################
Function InstallFirewall_MSSQL()                        # Function to install the Firewall Rule for the Microsoft SQL Database Server
{
	# Add the firewall rule to open Port 1433 for inbound management of the Microsoft SQL Server only if it doesn't exist
	if ( !(Get-NetFirewallRule | where DisplayName -EQ "Microsoft SQL Server - Port 1433") ) {
		Write-Host "`tOpening port 1433 for the Microsoft SQL Database Server on the Firewall" -ForegroundColor Cyan
		(New-NetFirewallRule -DisplayName "Microsoft SQL Server - Port 1433" -Direction Inbound –LocalPort 1433 -Protocol TCP -Action Allow) | Out-Null
		Write-Host "`t Firewall rule added successfully" -ForegroundColor Green
	}
}


####################################################################################################################################################################################
function Get-ExchangeOrgName{           # Function to get the Exchange Organisation Name
	$domain = [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain() 
	$ddn = $domain.GetDirectoryEntry().distinguishedName 
	$config = [ADSI]"LDAP://CN=Microsoft Exchange,CN=Services,CN=Configuration,$ddn" 
	$orgName = $config.psbase.children | where {$_.objectClass -eq 'msExchOrganizationContainer'} 
	$orgName.name 
}


####################################################################################################################################################################################
Function CheckExchangeDatabase()        # Function to Get and Check Exchange Databases and Mailboxes exist
{
	Param(
		[string]$DatabaseName,  # Specify the Database Name to check against
		[string]$DBname_Like,   # Specify a Search Query to check the Database Name against
		[string]$User_Exists,   # Check if a Exchange Mailbox exists
		[string]$User_Database, # Check if a Exchange Mailbox exists and return the Database Name where it is located
		[switch]$ThisServer     # Get the Database Name on this server
	)
	# Load the PowerShell Exchange Snapin
	add-pssnapin *exchange* -erroraction SilentlyContinue
	# Search the Exchange Servers Database list to see if the Database Name exists
	if ($DatabaseName) { 
		foreach ($Name in ((Get-MailboxDatabase).name)) {
			if ($Name -eq $DatabaseName) { $true }
		}
	# Search the Exchange Servers Database list to see if the Database Name exists based upon the search query
	}elseif ($DBname_Like) {
		foreach ($Name in ((Get-MailboxDatabase).name)) {
			if ($Name -like $DBname_Like) { $Name }
		}
	# Check if a Mailbox exists based upon the mailbox name
	}elseif ($User_Exists) {
		if ((Get-mailbox -identity "$User_Exists" -ErrorAction SilentlyContinue) -ne $null) {
			$true
		}
	# Check if a Mailbox exists based upon the mailbox name and return the Mailbox Database Name
	}elseif ($User_Database) {
		if (Get-mailbox -identity "$User_Database" -ErrorAction SilentlyContinue) {
			((Get-mailbox -identity "$User_Database" -ErrorAction SilentlyContinue).Database.Name)
		}
	# Search the Exchange Servers and list all existing Database Names
	}elseif ($ThisServer) {
		((Get-MailboxDatabase -Server $env:COMPUTERNAME).name)
	}else{
		((Get-MailboxDatabase).name)
	}
}


####################################################################################################################################################################################
Function InstallExchangeCAS()           # Function to install the features required for the Microsoft Exchange CAS Server Role
{
	Write-Host "`tInstalling the features required for Microsoft Exchange Server (CAS Role)" -ForegroundColor Cyan
	(Add-WindowsFeature -Name AS-HTTP-Activation, Web-Request-Monitor, Web-Basic-Auth, Web-Digest-Auth, Web-Dyn-Compression, Web-Lgcy-Mgmt-Console, Web-Mgmt-Service, Web-Metabase, Web-WMI, NET-Framework-45-Features, Desktop-Experience, RSAT-Clustering, RSAT-ADDS, RPC-over-HTTP-proxy, Windows-Identity-Foundation, WAS-Process-Model, Web-IP-Security, NLB, RSAT-NLB) | Out-Null
}


####################################################################################################################################################################################
Function InstallExchangeMBX()           # Function to install the features required for the Microsoft Exchange Mailbox Server Role
{
	Write-Host "`tInstalling the features required for Microsoft Exchange Server (Mailbox Role)" -ForegroundColor Cyan
	(Add-WindowsFeature -Name AS-HTTP-Activation, Web-Request-Monitor, Web-Basic-Auth, Web-Digest-Auth, Web-Dyn-Compression, Web-Lgcy-Mgmt-Console, Web-Mgmt-Service, Web-Metabase, Web-WMI, NET-Framework-45-Features, Desktop-Experience, RSAT-Clustering, RSAT-ADDS, RPC-over-HTTP-proxy, Windows-Identity-Foundation, WAS-Process-Model, RSAT-Clustering-CmdInterface) | Out-Null
}


####################################################################################################################################################################################
Function Exchange2016_PreCheck()  # Function to check Exchange 2016 Installation Options have all been set prior to installing
{
	# Check to see if the Exchange Installation Directory has already been set, if not then ask for one to be entered
	if (!$dExchangeInstal) {
		while ( ([string]::IsNullOrEmpty($dExchangeInstal)) -or (!(Test-Path "$dExchangeInstal")) ) {
			$script:dExchangeInstal = Read-Host "`n`tEnter the location of the Exchange setup files you would like to use for this installation"
		}
		if ($dExchangeInstal.Substring($dExchangeInstal.Length -1, 1) -ne "\") { $script:dExchangeInstal = "$dExchangeInstal\" }
		Write-Host "`t Exchange will be installed from `"$dExchangeInstal`"" -ForegroundColor Green
	}
	# Check if there are already any Exchange Servers on the domain
	if ( (([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -eq "") {
		# Check to see if the Exchange Organisation Name has already been set, if not then ask for one to be entered
		if (!$dExchangeOrgNme) {
			while ([string]::IsNullOrEmpty($dExchangeOrgNme)) {
				$script:dExchangeOrgNme = Read-Host "`n`tEnter the Exchange Organisation Name you would like to use for this installation"
			}
			Write-Host "`t Your Exchange Organisation Name will be `"$dExchangeOrgNme`"" -ForegroundColor Green
		}
		# Check if a new Organisational Unit needs to be created for SolidCP
		$choice = ""
		Write-Host "`n`tIt is reccomended that you have a Dedicated Organisational Unit for SolidCP" -ForegroundColor Yellow
		while ($choice -notmatch "[y|n]") {
			$choice = Read-Host "`n`t Do you wan to install a new Organisational Unit for SolidCP`? (Y/N)"
		}
		if ($choice -eq "y") {
			$dSolidCp_OU_Name = Read-Host "`n`t Please enter the name for the Organisational Unit"
			if (!($dSolidCp_OU_Name)) {$dSolidCp_OU_Name = "SolidCP"}
			#dCreateNewOU -Name "$dSolidCp_OU_Name" -Description "SolidCP Organisational Unit for Hosted Customers" -DiableInheritance -RemoveUserFromACL "S-1-5-32-554"
			dCreateNewOU -Name "$dSolidCp_OU_Name" -Description "SolidCP Organisational Unit for Hosted Customers"
		}
	}else{ # Otherwise get the Exchange Organisation Name from Active Directory
		$script:dExchangeOrgNme = Get-ExchangeOrgName
		Write-Host "`t Your Exchange Organisation Name is `"$dExchangeOrgNme`"" -ForegroundColor Green
	}
	# Ask for the Exchange Database Name on this server
	if (!$dExchangeDatabaseName) {
		while ([string]::IsNullOrEmpty($dExchangeDatabaseName)) {
			$script:dExchangeDatabaseName = Read-Host "`n`tEnter the Exchange Database Name you would like to use on this server"
		}
		Write-Host "`t The Exchange Database Name will be `"$dExchangeDatabaseName`"" -ForegroundColor Green
	}
}


####################################################################################################################################################################################
Function ExchangeMBX2016()              # Function to install the features required for the Microsoft Exchange 2016 Mailbox Server Role
{
	Write-Host "`tInstalling the features required for Microsoft Exchange 2016 Server (Mailbox Role)" -ForegroundColor Cyan
	(Add-WindowsFeature -Name AS-HTTP-Activation, Web-Request-Monitor, Web-Basic-Auth, Web-Digest-Auth, Web-Dyn-Compression, Web-Lgcy-Mgmt-Console, Web-Mgmt-Service, Web-Metabase, Web-WMI, NET-Framework-45-Features, Desktop-Experience, RSAT-Clustering, RSAT-ADDS, RPC-over-HTTP-proxy, Windows-Identity-Foundation, WAS-Process-Model, RSAT-Clustering-CmdInterface, RSAT-Clustering-Mgmt, RSAT-Clustering-PowerShell, RSAT-AD-PowerShell, GPMC) | Out-Null
	# Create the Exchange 2016 Pre-Requisites Directory in our Installation Files folder ready for downloading if it doesn't already exist
	if (!(Test-Path 'C:\_Install Files\Exchange 2016 Pre-Requisites')) { (md -Path 'C:\_Install Files\Exchange 2016 Pre-Requisites' -Force) | Out-Null }
	# Check if Microsoft .NET Framework v4.5.2 is installed, if not then download and install it
	if ( (Get-Itemproperty "HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full" | Select-Object -ExpandProperty "Release") -ge 379893 ) {}else{
		# Download the Microsoft .NET Framework v4.5.2
		Write-Host "`t Downloading the Exchange 2016 Pre-Requisites (Microsoft .NET Framework v4.5.2)" -ForegroundColor Green
		(New-Object System.Net.WebClient).DownloadFile("https://download.microsoft.com/download/E/2/1/E21644B5-2DF2-47C2-91BD-63C560427900/NDP452-KB2901907-x86-x64-AllOS-ENU.exe", "C:\_Install Files\Exchange 2016 Pre-Requisites\Microsoft .NET Framework 4.5.2- NDP452-KB2901907-x86-x64-AllOS-ENU.exe")
		# Install the Microsoft .NET Framework v4.5.2
		Write-Host "`t Installing the Pre-Requisites for Exchange 2016 (Microsoft .NET Framework v4.5.2)" -ForegroundColor Green
		(Start-Process -FilePath 'C:\_Install Files\Exchange 2016 Pre-Requisites\Microsoft .NET Framework 4.5.2- NDP452-KB2901907-x86-x64-AllOS-ENU.exe' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
	}
	# Check if Unified Communications Managed API 4.0 Runtime is installed, if not then download and install it
	if ( !(Test-Path 'C:\Program Files\Microsoft UCMA 4.0')) {
		# Download the Microsoft .NET Framework v4.5.2
		Write-Host "`t Downloading the Exchange 2016 Pre-Requisites (Unified Communications Managed API 4.0 Runtime)" -ForegroundColor Green
		(New-Object System.Net.WebClient).DownloadFile("https://download.microsoft.com/download/2/C/4/2C47A5C1-A1F3-4843-B9FE-84C0032C61EC/UcmaRuntimeSetup.exe", "C:\_Install Files\Exchange 2016 Pre-Requisites\Unified Communications Managed API 4.0 Runtime - UcmaRuntimeSetup.exe")
		# Install the Unified Communications Managed API 4.0 Runtime
		Write-Host "`t Installing the Exchange 2016 Pre-Requisites (Unified Communications Managed API 4.0 Runtime)" -ForegroundColor Green
		(Start-Process -FilePath 'C:\_Install Files\Exchange 2016 Pre-Requisites\Unified Communications Managed API 4.0 Runtime - UcmaRuntimeSetup.exe' -Argumentlist "/passive" -Wait -Passthru).ExitCode | Out-Null
	}
	# Check to see if the Exchange Installation options have been set
	Exchange2016_PreCheck
	# Check to see if Exchange is already installed on the domain
	if ( ((([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -le 15323) -and (!(Test-Path "C:\Program Files\Microsoft\Exchange Server\V15")) )  {
		# Extend the Active Directory Schema for Exchange 2016 installation
		Write-Host "`t Extending the Active Directory Schema" -ForegroundColor Green
		((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/PrepareSchema /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
		# If the Active Directory Schema has been extended for Exchange 2016 continue with preparing AD and install the first Mailbox Server
		if ( (([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -ge 15323) {
			# Prepare Active Directory for Exchange 2016 installation
			Write-Host "`t Preparing the Active Directory Schema" -ForegroundColor Green
			((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/PrepareAD /OrganizationName:`”$dExchangeOrgNme`” /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
			# Prepare Active Directory Domains for Exchange 2016 installation
			Write-Host "`t Preparing the Active Directory Domains" -ForegroundColor Green
			((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/PrepareAllDomains /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
			# Install the first Exchange 2016 Mailbox server on this machine
			Write-Host "`t Installing the first Exchange Mailbox Server`n" -ForegroundColor Green
			Write-Host "`t    ****************************************" -ForegroundColor Green
			Write-Host "`t    *                                      *" -ForegroundColor Green
			Write-Host "`t    *     This will take quite a while     *" -ForegroundColor Green
			Write-Host "`t    *           to fully complete          *" -ForegroundColor Green
			Write-Host "`t    *                                      *" -ForegroundColor Green
			Write-Host "`t    *  Please be patient while we install  *" -ForegroundColor Green
			Write-Host "`t    *      Exchange 2016 Mailbox Role      *" -ForegroundColor Green
			Write-Host "`t    *                                      *" -ForegroundColor Green
			Write-Host "`t    ****************************************" -ForegroundColor Green
			((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/mode:Install /role:Mailbox /OrganizationName:`"$dExchangeOrgNme`" /MdbName:`"$dExchangeDatabaseName`" /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
			# Install the Exchange 2016 Management Tools on this machine
			Write-Host "`t Installing the Exchange Management Tools" -ForegroundColor Green
			((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/Role:ManagementTools /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
		}
	# Install additional Exchange 2016 Mailbox Servers on the machine this is being run from
	}elseif ( ((([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -ge 15323) -and ((Get-ExchangeOrgName) -ne "" ) -and (!(Test-Path "C:\Program Files\Microsoft\Exchange Server\V15")) ) {
		# Install an additional Exchange 2016 Mailbox server on this machine
		Write-Host "`t Installing an additional Exchange Mailbox Server`n" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *     This will take quite a while     *" -ForegroundColor Green
		Write-Host "`t    *           to fully complete          *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    *  Please be patient while we install  *" -ForegroundColor Green
		Write-Host "`t    *      Exchange 2016 Mailbox Role      *" -ForegroundColor Green
		Write-Host "`t    *                                      *" -ForegroundColor Green
		Write-Host "`t    ****************************************" -ForegroundColor Green
		((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/mode:Install /role:Mailbox /OrganizationName:`"$dExchangeOrgNme`" /MdbName:`"$dExchangeDatabaseName`" /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
		# Install the Exchange 2016 Management Tools on this machine
		Write-Host "`t Installing the Exchange Management Tools" -ForegroundColor Green
		((Start-Process -FilePath "$dExchangeInstal\Setup.exe" -Argumentlist "/Role:ManagementTools /IAcceptExchangeServerLicenseTerms" -Wait -Passthru).ExitCode) | Out-Null
	}
}


####################################################################################################################################################################################
Function Exchange2016_PostCheck()  # Function to check Exchange 2016 Installation after installation
{
	# Check if there are already any Exchange 2016 Servers on the domain
	if ( (([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -ge "15254") {
		# Load the Microsoft Exchange PowerShell SnapIn
		add-pssnapin *exchange* -erroraction SilentlyContinue
		$dInstalledDBname = ((Get-MailboxDatabase -Server $env:COMPUTERNAME).name)
		Write-Host "`n`t Congratulations Microsoft Exchange has been successfully configured on this server" -ForegroundColor Green
		if ($dExchangeDatabaseName -eq  $dInstalledDBname){
			Write-Host "`t The Database Name on this server is `"$dExchangeDatabaseName`"" -ForegroundColor Green
		}else{
			Write-Host "`t The Database Name you entered was `"$dExchangeDatabaseName`"" -ForegroundColor Yellow
			Write-Host "`t The Database Name Installed on this server is `"$dInstalledDBname`"" -ForegroundColor Yellow
		}
		# Check if the Default Public Folder Mailbox is created
		if ( !((Get-Mailbox -PublicFolder).Name) ) {
			# Ask the user if they want to install a Public Folder Mailbox on this server
			$choice = ""
			Write-Host "`n`t You do not have any Public Folder Mailboxes installed" -ForegroundColor Yellow
			Write-Host "`t It is a requirement of SolidCP that at least 1 Public Folder Mailbox is configured" -ForegroundColor Yellow
			while ($choice -notmatch "[y|n]") {
				$choice = Read-Host "`n`t Do you want to install a Public Folder Mailbox on this server`? (Y/N)"
			}
			if ($choice -eq "y") {
				$dPublicFolderDatabase = Read-Host "`n`t Please enter the name for the Public Folder Mailbox"
				if (!($dPublicFolderDatabase)) {$dPublicFolderDatabase = "SolidCP-PF-Mailbox"}
				Write-Host "`t The Public Folder Database will be called `"$dPublicFolderDatabase`"" -ForegroundColor Green
				# Check if the Exchange Organisation Name is set, if not then get it
				if ( !($dExchangeOrgNme) ) { $dExchangeOrgNme = ([ADSI]("LDAP://CN=Microsoft Exchange,CN=Services," + (Get-ADRootDSE).configurationNamingContext)).Children[0].name }
				# Reset the Default Public Folder Mailbox Primary Hierarchy if it has not been deleted corectly
				$dDNquery = ("CN=$dExchangeOrgNme,CN=Microsoft Exchange,CN=Services," + (Get-ADRootDSE).configurationNamingContext) 
				if ( (([ADSI]("LDAP://$dDNquery")).msExchDefaultPublicFolderMailbox) -split ";" -contains "00000000-0000-0000-0000-000000000000") {
					$orgName = ([ADSI]("LDAP://$dDNquery"))
					$orgName.PutEx(1,"msExchDefaultPublicFolderMailbox",0)
					$orgName.SetInfo()
				}
				# Create the Default Public Folder Mailbox for Primary Hierarchy
				(New-Mailbox -Name "$dPublicFolderDatabase" –PublicFolder) | Out-Null
			} else {
				write-host "`n`t You MUST install a Public Folder Mailbox for SolidCP to function correctly" -ForegroundColor Yellow
			}
		}
		if ( (Test-Path "C:\Program Files (x86)\SolidCP Installer") -and  (Test-Path "C:\Program Files\Microsoft\Exchange Server\V15") ) {
			# Add the SCPServer User to the Domain Groups called "Enterprise Admins" and "Organization Management"
			(AddDomainUserToDomainGroup "$dLangDomainEnterpriseAdmins" "SCPServer-$env:computerName") | Out-Null    # Add the "SCPServer-[ComputerName]" User to the group called "Enterprise Admins"
			(AddDomainUserToDomainGroup "$env:USERDOMAIN\Organization Management" "SCPServer-$env:computerName") | Out-Null         # Add the "SCPServer-[ComputerName]" User to the group called "Organization Management"
		}
		# Check to make sure the default options for SolidCP are configured corectly
		Exchange2016_UpgradeCheck
		# Prompt user to reboot this machine
		Write-Host "`n`t You must reboot this machine before you can use this Exchange Server" -ForegroundColor Green
		(dRebootConfirm) | Out-Null
	}
}


####################################################################################################################################################################################
Function Exchange2016_UpgradeCheck()  # Function to check Exchange 2016 Installation after Upgrades
{
	# Check if there are already any Exchange 2016 Servers on the domain
	if ( ((([ADSI]("LDAP://CN=ms-Exch-Schema-Version-Pt," + ([ADSI]"LDAP://RootDSE").schemaNamingContext)).rangeUpper) -ge "15254") -and (Test-Path "C:\Program Files\Microsoft\Exchange Server\V15") ) {
		# Load the Microsoft Exchange PowerShell SnapIn
		add-pssnapin *exchange* -erroraction SilentlyContinue
		# Enable Automatic Replies
		if ((Get-RemoteDomain Default).AutoReplyEnabled -eq $false) {
			Write-Host "`t Enabling Auto Replies on the Exchange Server" -ForegroundColor Green
			Set-RemoteDomain Default -AutoReplyEnabled $true
		}
		# Enable Automatic Forwards
		if ((Get-RemoteDomain Default).AutoForwardEnabled -eq $false) {
			Write-Host "`t Enabling Auto Forwards on the Exchange Server" -ForegroundColor Green
			Set-RemoteDomain Default –AutoForwardEnabled $true
		}
		# Enable OOF for Outlook 2003 and previous (for Exchange 2007 and 2010 support)
		if ((Get-RemoteDomain Default).AllowedOOFType -eq "External") {
			Write-Host "`t Enabling Legacy Out Of Office Types on the Exchange Server" -ForegroundColor Green
			Set-RemoteDomain Default –AllowedOOFType "ExternalLegacy"
		}
		# Enable Meeting Forward Notifications
		if ((Get-RemoteDomain Default).MeetingForwardNotificationEnabled -eq $false) {
			Write-Host "`t Enabling Meeting Forward Notifications on the Exchange Server" -ForegroundColor Green
			Set-RemoteDomain Default -MeetingForwardNotificationEnabled $true
		}
		# Check if there are already any Exchange 2016 Servers on the domain
	}
}


####################################################################################################################################################################################
Function InstallExchangeFSW()           # Function to install the features required for the Microsoft Exchange File Share Witness Server Role
{
	Write-Host "`tInstalling the features required for Microsoft Exchange Server (File Share Witness Role)" -ForegroundColor Cyan
	(Add-WindowsFeature -Name FS-FileServer) | Out-Null
}


####################################################################################################################################################################################
Function InstallMailEnable()           # Function to install the features required for the MailEnable Email Server
{
	Write-Host "`tInstalling the features required for MailEnable Email Server" -ForegroundColor Cyan
	(Add-WindowsFeature -Name Web-Request-Monitor, Web-Basic-Auth, Web-Metabase, Web-WMI, WAS-Process-Model, Web-ASP, Web-CGI, Web-Mgmt-Compat, Web-Lgcy-Mgmt-Console, RSAT, WAS, WAS-NET-Environment, WAS-Config-APIs, Web-Basic-Auth, Web-IP-Security) | Out-Null
	# Create the MailEnable Directory in our Installation Files folder ready for downloading if it doesn't already exist
	if (!(Test-Path 'C:\_Install Files\MailEnable')) { (md -Path 'C:\_Install Files\MailEnable' -Force) | Out-Null }
	# Download the files required for MailEnable
	Write-Host "`t Downloading the latest version of MailEnable" -ForegroundColor Green
	(New-Object System.Net.WebClient).DownloadFile("https://www.mailenable.com/Standard914.exe", "C:\_Install Files\MailEnable\MailEnable-Standard914.exe")
	# Install the MailEnable Email Server
	Write-Host "`t Installing the MailEnable Email Server" -ForegroundColor Green
	(Start-Process -FilePath 'C:\_Install Files\MailEnable\MailEnable-Standard914.exe' -Argumentlist "/s" -Wait -Passthru).ExitCode | Out-Null
	Write-Host "`t Configuring the MailEnable Email Server" -ForegroundColor Green

	# This section still needs to be written once i have heard back from MailEnable on how to use the Silent Install options
}


####################################################################################################################################################################################
Function InstallAwStats()               # Function for downloading and installing AwStats
{
	if ( ( !(Test-Path "C:\AWStats\wwwroot\") ) -and ( !(Test-Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats") ) ) { 
		Write-Host "`tInstalling the features required for AwStats" -ForegroundColor Cyan
		# Install the additional IIS Requirements for AwStats
		(Add-WindowsFeature -Name Web-CGI) | Out-Null
		# Configure all logging options to be enabled for websites in IIS
		Set-WebConfigurationProperty -Filter System.Applicationhost/Sites/SiteDefaults/logfile -Name LogExtFileFlags -Value "Date,Time,ClientIP,UserName,SiteName,ComputerName,ServerIP,Method,UriStem,UriQuery,HttpStatus,Win32Status,BytesSent,BytesRecv,TimeTaken,ServerPort,UserAgent,Cookie,Referer,ProtocolVersion,Host,HttpSubStatus"
		# Create the AwStats Directory in our Installation Files folder ready for downloading if it doesn't already exist
		if (!(Test-Path 'C:\_Install Files\AwStats')) { (md -Path 'C:\_Install Files\AwStats' -Force) | Out-Null }
		# Download the files required for AwStats
		Write-Host "`t Downloading the latest version of AwStats" -ForegroundColor Green
		(New-Object System.Net.WebClient).DownloadFile("https://github.com/eldy/awstats/archive/develop.zip", "C:\_Install Files\AwStats\awstats.zip")
		# Get the highest current SolidCP Version from the Installer XML File from the SolidCP Installation Website
		[string]$dSolidCPversion = ((([xml](New-Object System.Net.WebClient).DownloadString("http://installer.solidcp.com/Data/ProductReleasesFeed-1.0.xml")).components.component.releases.release).version | measure -Maximum).Maximum
		# Download the latest SolidCP AwStats Viewer based upon the highest version above
		Write-Host "`t Downloading the latest version of the AwStats Viewer from the SolidCP website" -ForegroundColor Green
		(New-Object System.Net.WebClient).DownloadFile("http://installer.solidcp.com/Files/stable/Tools/SolidCP-AWStatsViewer-$dSolidCPversion.zip", "C:\_Install Files\AwStats\SolidCP-AWStatsViewer-$dSolidCPversion.zip")
		# Unzip the downloaded files to the C:\AwStats directory
		Write-Host "`t Extracting and installing AwStats from the downloaded files" -ForegroundColor Green
		Add-Type -assembly "system.io.compression.filesystem"
		[io.compression.zipfile]::ExtractToDirectory("C:\_Install Files\AwStats\awstats.zip", "C:\AWStats")
		[io.compression.zipfile]::ExtractToDirectory("C:\_Install Files\AwStats\SolidCP-AWStatsViewer-$dSolidCPversion.zip", "C:\AWStats\awstats-develop\wwwroot")
		( (Get-ChildItem "C:\AWStats\" -include *.cvsignore -recurse | foreach ($_) {remove-item $_.fullname}) ) | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\wwwroot\LICENSE.txt" "C:\AWStats") | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\wwwroot\Readme.htm" "C:\AWStats") | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\wwwroot\Web-v2.0.config" "C:\AWStats") | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\docs\*"    "C:\AwStats\docs\") | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\tools\*"   "C:\AwStats\tools\") | Out-Null
		(Move-Item "C:\AWStats\awstats-develop\wwwroot\*" "C:\AwStats\wwwroot\") | Out-Null
		(Remove-Item "C:\AWStats\awstats-develop" -Recurse) | Out-Null
		(md -Path  "C:\AWStats\wwwroot\bin\" -Force) | Out-Null
		(Move-Item "C:\AWStats\wwwroot\*.dll" "C:\AwStats\wwwroot\bin\") | Out-Null
		(Move-Item "C:\AWStats\wwwroot\SolidCP.AWStats.Viewer.dll.config" "C:\AwStats\wwwroot\bin\") | Out-Null
		(md -Path 'C:\AWStats\data') | Out-Null
		(md -Path 'C:\AWStats\logs') | Out-Null

		# Create the AwStats configuration File Template file
		(New-Item C:\AWStats\AwStats-Configuration-Template-File.txt -type file -force -value "LogFormat = `"%time2 %other %other %other %method %url %other %other %logname %host %other %ua %other %referer %other %code %other %other %bytesd %other %other`"`r`nLogSeparator = `" `"`r`nDNSLookup = 1`r`nDirCgi = `"/awstats/cgi-bin`"`r`nDirIcons = `"/awstats/icon`"`r`nAllowFullYearView=3`r`nAllowToUpdateStatsFromBrowser = 0`r`nUseFramesWhenCGI = 1`r`nShowFlagLinks = `"en fr de it nl es`"`r`nLogFile = `"[LOGS_FOLDER]\u_ex%YY-3%MM-3%DD-3.log`"`r`nDirData = `"C:\AWStats\data`"`r`nSiteDomain = `"[DOMAIN_NAME]`"`r`nHostAliases = [DOMAIN_ALIASES]`r`n") | Out-Null
		# Create the BAT File that will be used for AwStats to run the processing of website stats
		(New-Item C:\AWStats\UpdateStats.bat -type file -force) | Out-Null
		# Create the AwStats Scheduled Job as XML Format so we can import it
		(New-Item C:\AWStats\AwStats-Scheduled-Task-Import.xml -type file -force -value "<?xml version=`"1.0`" encoding=`"UTF-16`"?>`r`n<Task version=`"1.2`" xmlns=`"http://schemas.microsoft.com/windows/2004/02/mit/task`">`r`n  <RegistrationInfo>`r`n    <Date>2015-01-01T00:00:00.000000</Date>`r`n    <Author>BUILTIN\$dLangAdministratorName</Author>`r`n    <Description>Batch file that runs every 1 hour to poll each website that has Advanced Stats enabled on it.</Description>`r`n  </RegistrationInfo>`r`n  <Triggers>`r`n    <CalendarTrigger>`r`n      <Repetition>`r`n        <Interval>PT1H</Interval>`r`n        <Duration>P1D</Duration>`r`n        <StopAtDurationEnd>false</StopAtDurationEnd>`r`n      </Repetition>`r`n      <StartBoundary>2015-01-01T00:00:00</StartBoundary>`r`n      <ExecutionTimeLimit>PT1H</ExecutionTimeLimit>`r`n      <Enabled>true</Enabled>`r`n      <ScheduleByDay>`r`n        <DaysInterval>1</DaysInterval>`r`n      </ScheduleByDay>`r`n    </CalendarTrigger>`r`n  </Triggers>`r`n  <Principals>`r`n    <Principal id=`"Author`">`r`n      <RunLevel>HighestAvailable</RunLevel>`r`n      <GroupId>S-1-5-32-544</GroupId>`r`n    </Principal>`r`n  </Principals>`r`n  <Settings>`r`n    <MultipleInstancesPolicy>IgnoreNew</MultipleInstancesPolicy>`r`n    <DisallowStartIfOnBatteries>false</DisallowStartIfOnBatteries>`r`n    <StopIfGoingOnBatteries>false</StopIfGoingOnBatteries>`r`n    <AllowHardTerminate>true</AllowHardTerminate>`r`n    <StartWhenAvailable>true</StartWhenAvailable>`r`n    <RunOnlyIfNetworkAvailable>false</RunOnlyIfNetworkAvailable>`r`n    <IdleSettings>`r`n      <StopOnIdleEnd>true</StopOnIdleEnd>`r`n      <RestartOnIdle>false</RestartOnIdle>`r`n    </IdleSettings>`r`n    <AllowStartOnDemand>true</AllowStartOnDemand>`r`n    <Enabled>true</Enabled>`r`n    <Hidden>false</Hidden>`r`n    <RunOnlyIfIdle>false</RunOnlyIfIdle>`r`n    <WakeToRun>false</WakeToRun>`r`n    <ExecutionTimeLimit>PT12H</ExecutionTimeLimit>`r`n    <Priority>7</Priority>`r`n    <RestartOnFailure>`r`n      <Interval>PT1M</Interval>`r`n      <Count>10</Count>`r`n    </RestartOnFailure>`r`n  </Settings>`r`n  <Actions Context=`"Author`">`r`n    <Exec>`r`n      <Command>C:\AWStats\UpdateStats.bat</Command>`r`n      <WorkingDirectory>C:\AWStats</WorkingDirectory>`r`n    </Exec>`r`n  </Actions>`r`n</Task>`r`n") | Out-Null
		# Create the task and import the settings from the XML file we created above
		(schtasks /create /XML C:\AWStats\AwStats-Scheduled-Task-Import.xml /tn "AwStats - Update Stats") | Out-Null
		# Change the bindings on the default website to Port 8080 so we can install AwStats on Port 80 unless it has already been done
		if ( (Get-WebBinding -Name 'Default Web Site').bindingInformation -eq "*:80:" ) {
			Set-WebBinding -Name 'Default Web Site' -BindingInformation "*:80:" -PropertyName Port -Value 8080
			# Another way to do the above is as per the line below
			#Set-WebConfigurationProperty "/system.applicationHost/sites/site[@name='Default Web Site']/bindings/binding[@protocol='http']" -name bindingInformation -value '*:8080:'
		}
		# Create the AppPool for AwStats
		Write-Host "`t Creating the AwStats Application Pool in IIS" -ForegroundColor Green
		(New-WebAppPool -Name "AwStats .NET v4" -Force) | Out-Null
		# Create the new website for AwStats
		Write-Host "`t Creating the AwStats website in IIS" -ForegroundColor Green
		(New-Website -Name "AwStats" -Port 80 -IPAddress "*" -PhysicalPath "C:\AWStats\wwwroot" -ApplicationPool "AwStats .NET v4" -Force) | Out-Null
		(Set-ItemProperty "IIS:\Sites\AwStats" -name logfile.directory "C:\AWStats\logs" -Force) | Out-Null
		# Grant Read & Execute permissions on wwwroot folder and Logs Folder
		SetAccessToFolderNoChk "C:\AWStats\wwwroot" "IIS AppPool\AwStats .NET v4" "ReadAndExecute" "Allow";
		SetAccessToFolderNoChk "C:\AWStats\wwwroot" "BUILTIN\IIS_IUSRS" "ReadAndExecute" "Allow";
		SetAccessToFolderNoChk "C:\AWStats\wwwroot" "BUILTIN\$dLangUsersGroup" "ReadAndExecute" "Allow";
		SetAccessToFolderNoChk "C:\AWStats\logs" "NT SERVICE\TrustedInstaller" "FullControl" "Allow";
		(Restart-Service 'World Wide Web Publishing Service' -Force -WarningAction SilentlyContinue) | Out-Null
		Sleep 5
		Write-Host "`t Adding the Registry Entries for AwStats" -ForegroundColor Green
		InstallAwStatsRegKeys | Out-Null
	}
}

####################################################################################################################################################################################
Function InstallAwStatsRegKeys()        # Function for downloading and installing AwStats
{
	if (!(Test-Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats")) {
		(New-Item -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Force) ; sleep 2
	}
	if (Test-Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats") {
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "DisplayName" -Value "AWStats"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "UninstallString" -Value "C:\AWStats\uninstall.exe"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "Publisher" -Value "Laurent Destailleur"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "DisplayIcon" -Value "C:\AWStats\docs\awstats.ico"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "URLInfoAbout" -Value "http://awstats.sourceforge.net"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "Comments" -Value "copyright 2000/2015 Laurent Destailleur"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "HelpLink" -Value "http://awstats.sourceforge.net/docs/index.html"
		New-ItemProperty -Path "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\AWStats" -Name "URLUpdateInfo" -Value "http://awstats.sourceforge.net"
	}
}


####################################################################################################################################################################################
function CopyAndFormatXMLfile($filePath)    # Function to copy file and format the file into standard XML format so we can process it consistently
{
	[xml]$xml = Get-Content -Path "$filePath"
	# Save the file
	$xml.Save((split-path $filePath) + "\cust_" + (split-path $filePath -Leaf));
}


####################################################################################################################################################################################
function CheckXMLnode ($dFilePath, $dXMLNode, $dXMLname, $dXMLvalue) # Function to check if a node exists in an XML file with specific values
{ # Usage - CheckXMLnode $dFilePath "//SecurityClasses" "SecurityClass" "name-Custom"
  # Usage - CheckXMLnode $dFilePath "//SecurityClass" "Name" "Custom"
	[xml]$xml = Get-Content -Path "$dFilePath"
	if ([string]::IsNullOrEmpty($dXMLvalue)) {
		if ($xml.selectNodes("$dXMLNode").$dXMLname) { $true } else { $false }
	}else{
		if ($xml.selectNodes("$dXMLNode").$dXMLname -contains "$dXMLvalue") { $true } else { $false }
	}
}


####################################################################################################################################################################################
function ModifyXML([String] $dFilePath, $dAction, [String] $dNodePath, $dElement, $dValue) # Function to Add, Comment  or UnComment XML nodes in an XML File
{ # Usage - ModifyXML $dFilePath "Add" "//TestParent" "TestChild" "TestValue"
  # Usage - ModifyXML $dFilePath "Add" "//TestParent" "TestChild" @("Attribute","Value")
  # Usage - ModifyXML $dFilePath "Add" "//TestParent" "TestChild" @( ("Attribute1","Value 1"), ("Attribute2","Value 2"), ("Attribute3","Value 3"), ("Attribute4","Value 4"), ("Attribute5","Value 5") )
  # Usage - ModifyXML $dFilePath "Delete" "//TestParent/TestChild"
  # Usage - ModifyXML $dFilePath "Delete" "//TestChild[@TestAttribute]"
  # Usage - ModifyXML $dFilePath "Update" "//TestChild" "Updated Value"
  # Usage - ModifyXML $dFilePath "Update" "//TestChild[@TestAttribute='Original Value']/@TestAttribute" "New Value"
  # Usage - ModifyXML $dFilePath "Get" "//TestParent/TestChild"
  # Usage - ModifyXML $dFilePath "Get" "//TestChild[@TestAttribute]/@TestAttribute"
  # Usage - ModifyXML $dFilePath "Comment" "//TestParent/TestChild"
  # Usage - ModifyXML $dFilePath "Comment" "//TestChild[@TestAttribute='Value']"
  # Usage - ModifyXML $dFilePath "UnComment" "/TestChild"
  # Usage - ModifyXML $dFilePath "UnComment" 'TestAttribute="Value"'
	[xml]$xml = Get-Content -Path "$dFilePath"
	if ($dAction -eq "Add") {
		$Child = $xml.CreateElement("$dElement")
		if ($dValue -is [System.Array]) {
			if ($dValue[0] -is [System.Array]) { # The Attribute are in a Multi Dimensional Array
				$dCheckNode = "$dNodePath/$dElement"
				if ( !(CheckXMLnode "$dFilePath" "$dCheckNode" $dValue[0][0] $dValue[0][1]) ) {
					foreach($value in $dValue) {
						$Child.SetAttribute($value[0],$value[1])
						$xml.selectNodes("$dNodePath").AppendChild($Child) | Out-Null
					}
				}
			}else{ # The Attributes are in a Single Level Array
				$dCheckNode = "$dNodePath/$dElement"
				if ( !(CheckXMLnode "$dFilePath" "$dCheckNode" $dValue[0] $dValue[1]) ) {
					$Child.SetAttribute($dValue[0],$dValue[1])
					$xml.selectNodes("$dNodePath").AppendChild($Child) | Out-Null
				}
			}
		}
		elseif ($dValue -isnot [System.Array]) { # Simple Element with a Value
			if ( !(CheckXMLnode "$dFilePath" "$dNodePath" "$dElement" "$dValue") ) {
				$Child.InnerText = "$dValue"
				$xml.selectNodes("$dNodePath").AppendChild($Child) | Out-Null
			}
		}
		$xml.Save("$dFilePath") | Out-Null
	}elseif ($dAction -eq "Update") {
		$dNode = $xml.SelectNodes($dNodePath)
		foreach ($node in $dNode) {
			if ($node -ne $null) {
				if ( $node.NodeType -eq "Element") { $node.InnerXml = $dElement }
				else { $node.Value = $dElement }
			}
		}
		$xml.Save("$dFilePath") | Out-Null
	}elseif ($dAction -eq "Get") {
		$dNode = $xml.SelectNodes($dNodePath)
		if ($dNode -ne $null) {
			if ( $dNode.NodeType -eq "Element") { $dNode.InnerXml }
			else { $dNode.Value }
		}
	}elseif ($dAction -eq "Delete") {
		if ( ($xml.selectNodes("$dNodePath")).ParentNode.Name ) {
			($xml | select-xml -xpath ("//" + ($xml.selectNodes("$dNodePath")).ParentNode.Name) | ForEach-Object {$_.node.removechild((select-xml -xpath $dNodePath  -xml $xml).node)}) | Out-Null
			$xml.Save("$dFilePath");
		}
	}elseif ($dAction -eq "Comment") {
		$xml.SelectNodes("$dNodePath") | ForEach-Object {
			$Comment = $xml.CreateComment($_.OuterXml);
			$_.ParentNode.ReplaceChild($Comment, $_) | Out-Null
		}
		$xml.Save("$dFilePath");
	}elseif ($dAction -eq "UnComment") {
		$xml.SelectNodes("//comment()") | ForEach-Object {     
			($_.InnerText | convertto-xml).SelectNodes("/descendant::*[contains(text(), '$dNodePath')]") | ForEach-Object { 
				$UnComment = $_;
				(Get-Content "$dFilePath") | ForEach-Object { $_.Replace("<!--" + $UnComment.InnerText + "-->", $UnComment.InnerText) } | Set-Content "$dFilePath"
			}
		}
	}
}


####################################################################################################################################################################################
Function FixServer2012UAC()                 # Function to fix the UAC bug on Server 2012 R2 to make the server secure
{
	Write-Host "`tFixing the Server 2012 R2 UAC bug on this machine" -ForegroundColor Cyan
	# Create a local group called "Administrators File Access" on the serverif it does not exist
	CreateLocalUserOrGroup "Administrators File Access" "Local Administrators File Access - SCP Harden IIS  ********* DO NOT DELETE *********" "Group"
	# Add the Local "Administrator" to the Local Group called "Administrators File Access" unless it is already a mmember of the group
	AddLocalUserToLocalGroup "Administrators File Access" "$dLangAdministratorName"
	# If the machine is a member of a domain then add the "Domain Admins" group to the Local Group called "Administrators File Access" unless it is already a mmember of the group
	if ($dDomainMember) {AddDomainUserToLocalGroup "Administrators File Access" "$dLangDomainAdminsGroup"}
	# Create the "C:\HostingSpaces" folder if it doesnt exist
	if(!(Test-Path $env:SystemDrive"\HostingSpaces")) {
		(New-Item $env:SystemDrive"\HostingSpaces" -itemType directory) | Out-Null
	}
	# Take ownership of the "C:\HostingSpaces" directory and add the "Administrators" and "Administrators File Access" Groups (Full Access) and Remove inheritance
	if(Test-Path $env:SystemDrive"\HostingSpaces") {
		# Take owenership of the "C:\HostingSpaces" folder with the Local "Administrators" Group
		Write-Host "`t Taking ownership of the HostingSpaces directory" -ForegroundColor Green
		Invoke-Command {takeown /A /F C:\HostingSpaces /R /D Y} | Out-Null;
		# Grant Full Access to the "c:\HostingSpaces" folder for the local group "Administrators"
		Write-Host "`t Granting access to the HostingSpaces directory for the Administrators group" -ForegroundColor Green
		SetAccessToFolderNoChk "C:\HostingSpaces" "$dLangAdministratorGroup" "FullControl" "Allow";
		# Grant Full Access to the "c:\HostingSpaces" folder for the local group "Administrators File Access"
		Write-Host "`t Granting access to the HostingSpaces directory for the Administrators File Access group" -ForegroundColor Green
		SetAccessToFolderNoChk "C:\HostingSpaces" "Administrators File Access" "FullControl" "Allow";
		# Grant Full Access to the "c:\HostingSpaces" folder for the domain group "Domain Admins"
		if ($dDomainMember) {Write-Host "`t Granting access to the HostingSpaces directory for the Domain Admins group" -ForegroundColor Green}
		if ($dDomainMember) {SetAccessToFolderNoChk "C:\HostingSpaces" "$dLangDomainAdminsGroup" "FullControl" "Allow";}
		# Disable inheritance on the "c:\HostingSpaces" folder
		Write-Host "`t Disabling Inheritance on the HostingSpaces directory" -ForegroundColor Green
		DisableFolderInheritance "C:\HostingSpaces";
		# Remove "Allow" permissions to the "C:\HostingSpaces" folder for the "Users", "CREATOR OWNER" and "SYSTEM" Groups
		Write-Host "`t Removing permissions on the HostingSpaces directory" -ForegroundColor Green
		RemoveAccessToFolder "C:\HostingSpaces" "$dLangUsersGroup" "FullControl" "Allow";
		RemoveAccessToFolderNoChk "C:\HostingSpaces" "$dLangCreatorOwnerName" "FullControl" "Allow";
		RemoveAccessToFolderNoChk "C:\HostingSpaces" "$dLangSystemName" "FullControl" "Allow";
		# Remove Special permissions to the "C:\HostingSpaces" folder for the "Administrator" User as it has access through the Administrators group
		RemoveAccessToFolderNoChk "C:\HostingSpaces" "$dLangAdministratorName" "FullControl" "Allow";
	}
	# Remove "Allow" permissions to the "C:\" drive to the "Users" Group
	Write-Host "`t Removing access to the C Drive for the Users group" -ForegroundColor Green
	RemoveAccessToFolder "C:\" "$dLangUsersGroup" "FullControl" "Allow";
	# Grant permissions for the "C:\" drive to the "Local Service" and "Network Service" Groups (Read & Execute, List Folder Contents, Read)
	Write-Host "`t Allowing access to the C Drive for the Local Service and Network Service groups" -ForegroundColor Green
	SetAccessToFolderNoChk "C:\" "$dLangLocalServiceName" "ReadAndExecute" "Allow";
	SetAccessToFolderNoChk "C:\" "$dLangNetworkServiceName" "ReadAndExecute" "Allow";
	# Grant Full Access for the "C:\" drive for the local group "Administrators File Access"
	Write-Host "`t Allowing access to the C Drive for the Administrators File Access group" -ForegroundColor Green
	SetAccessToFolderNoChk "C:\" "Administrators File Access" "FullControl" "Allow";
	# Grant Full Access for the "C:\" drive for the domain group "Domain Admins"
	if ($dDomainMember) {Write-Host "`t Allowing access to the C Drive for the Domain Admins group" -ForegroundColor Green}
	if ($dDomainMember) {SetAccessToFolderNoChk "C:\" "$dLangDomainAdminsGroup" "FullControl" "Allow";}
	# Take ownership for *.exe files in the "C:\Windows\System32" and "C:\Windows\SysWOW64" directories (EXE Files ONLY)
	#FixServer2012UAC_1  # Commented out as we are still testing the SQL Backups needing permissions to some files
	# Deny permissions for *.exe files in the "C:\Windows\System32" folder for the "IIS_IUSRS" Group (Deny All)
	#FixServer2012UAC_2  # Commented out as we are still testing the SQL Backups needing permissions to some files
	# Copy over the members of the Local "Administrators" group over to the Local "Administrators File Access" group
	CopyGroupMembers "Administrators" "Administrators File Access" "Local"
}


####################################################################################################################################################################################
Function FixServer2012UAC_1()               # Function to Take Ownership of *.exe files in the System32 and SysWOW64 folders
{
	# Take ownership for *.exe files in the "C:\Windows\System32" and "C:\Windows\SysWOW64" directories (EXE Files ONLY)
	Write-Host "`t Taking ownership for all executables files in the System32 directory" -ForegroundColor Green
	Invoke-Command {takeown /A /F c:\windows\system32\*.exe} | Out-Null;
	Write-Host "`t Taking ownership for all executables files in the SysWOW64 directory" -ForegroundColor Green
	Invoke-Command {takeown /A /F c:\windows\SysWOW64\*.exe} | Out-Null;
}


####################################################################################################################################################################################
Function FixServer2012UAC_2()               # Function to Deny permissions for *.exe files in the "C:\Windows\System32" folder for the "IIS_IUSRS" Group (Deny All)
{
	# Deny permissions for *.exe files in the "C:\Windows\System32" folder for the "IIS_IUSRS" Group (Deny All)
	Write-Host "`t Deny access for all executables files in the System32 directory for the IIS_IUSRS group" -ForegroundColor Green
	Invoke-Command {cacls c:\windows\system32\*.exe /E /D IIS_IUSRS} | Out-Null;
	# Deny permissions for *.exe files in the "C:\Windows\SysWOW64" folder for the "IIS_IUSRS" Group (Deny All)
	Write-Host "`t Deny access for all executables files in the SysWOW64 directory for the IIS_IUSRS group" -ForegroundColor Green
	Invoke-Command {cacls c:\windows\SysWOW64\*.exe /E /D IIS_IUSRS} | Out-Null;
}


####################################################################################################################################################################################
Function FixServer2012UAC_2_UNDO()          # Function to REMOVE the Deny permissions for *.exe files in the "C:\Windows\System32" folder for the "IIS_IUSRS" Group (Deny All)
{
	# This will REVOKE the Deny permissions for *.exe files in the "C:\Windows\System32" folder for the "IIS_IUSRS" Group (Deny All)
	Invoke-Command {icacls  c:\windows\system32\*.exe /remove:d IIS_IUSRS} | Out-Null;
	# This will REVOKE the Deny permissions for *.exe files in the "C:\Windows\SysWOW64" folder for the "IIS_IUSRS" Group (Deny All)
	Invoke-Command {icacls  c:\windows\SysWOW64\*.exe /remove:d IIS_IUSRS} | Out-Null;
}


####################################################################################################################################################################################
Function HardenDotNETforIIS()               # Function to harden .NET for IIS to make the server secure
{
	Write-Host "`tHardening IIS on this machine" -ForegroundColor Cyan
	$dNET2_32_dir = "C:\Windows\Microsoft.NET\Framework\v2.0.50727\CONFIG"
	if (!(Test-Path "$dNET2_32_dir\cust_web_mediumtrust.config")) {   # .NET v2 (32 Bit)
		Write-Host "`t Hardening IIS for .NET v2 (32 Bit)" -ForegroundColor Green
		Copy-Item "$dNET2_32_dir\web_mediumtrust.config" "$dNET2_32_dir\cust_web_mediumtrust.config"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='EnvironmentPermission']"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='PrintingPermission']"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Add" "//SecurityClasses" "SecurityClass" @( ("Name","OleDbPermission"), ("Description","System.Data.OleDb.OleDbPermission, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089") )
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='EnvironmentPermission']"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='PrintingPermission']"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='WebPermission']"
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","WebPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET2_32_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","OleDbPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET2_32_dir\web.config" "Update" "//location[@allowOverride='true']/@allowOverride" "false"
		ModifyXML "$dNET2_32_dir\web.config" "Add" "//system.web/securityPolicy" "trustLevel" @( ("name","Custom"), ("policyFile", "cust_web_mediumtrust.config") )
	}
	$dNET2_64_dir = "C:\Windows\Microsoft.NET\Framework64\v2.0.50727\CONFIG"
	if (!(Test-Path "$dNET2_64_dir\cust_web_mediumtrust.config")) { # .NET v2 (64 Bit)
		Write-Host "`t Hardening IIS for .NET v2 (64 Bit)" -ForegroundColor Green
		Copy-Item "$dNET2_64_dir\web_mediumtrust.config" "$dNET2_64_dir\cust_web_mediumtrust.config"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='EnvironmentPermission']"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='PrintingPermission']"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Add" "//SecurityClasses" "SecurityClass" @( ("Name","OleDbPermission"), ("Description","System.Data.OleDb.OleDbPermission, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089") )
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='EnvironmentPermission']"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='PrintingPermission']"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='WebPermission']"
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","WebPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET2_64_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","OleDbPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET2_64_dir\web.config" "Update" "//location[@allowOverride='true']/@allowOverride" "false"
		ModifyXML "$dNET2_64_dir\web.config" "Add" "//system.web/securityPolicy" "trustLevel" @( ("name","Custom"), ("policyFile", "cust_web_mediumtrust.config") )
	}
	$dNET4_32_dir = "C:\Windows\Microsoft.NET\Framework\v4.0.30319\Config"
	if (!(Test-Path "$dNET4_32_dir\cust_web_mediumtrust.config")) {   # .NET v4 (32 Bit)
		Write-Host "`t Hardening IIS for .NET v4 (32 Bit)" -ForegroundColor Green
		Copy-Item "$dNET4_32_dir\web_mediumtrust.config" "$dNET4_32_dir\cust_web_mediumtrust.config"
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='EnvironmentPermission']"
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='PrintingPermission']"
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Add" "//SecurityClasses" "SecurityClass" @( ("Name","OleDbPermission"), ("Description","System.Data.OleDb.OleDbPermission, System.Data, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089") )
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='EnvironmentPermission']"
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='PrintingPermission']"
		ModifyXML "$dNET4_32_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","OleDbPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET4_32_dir\web.config" "Update" "//location[@allowOverride='true']/@allowOverride" "false"
		ModifyXML "$dNET4_32_dir\web.config" "Add" "//system.web/securityPolicy" "trustLevel" @( ("name","Custom"), ("policyFile", "cust_web_mediumtrust.config") )
	}
	$dNET4_64_dir = "C:\Windows\Microsoft.NET\Framework64\v4.0.30319\Config"
	if (!(Test-Path "$dNET4_64_dir\cust_web_mediumtrust.config")) { # .NET v4 (64 Bit)
		Write-Host "`t Hardening IIS for .NET v4 (64 Bit)" -ForegroundColor Green
		Copy-Item "$dNET4_64_dir\web_mediumtrust.config" "$dNET4_64_dir\cust_web_mediumtrust.config"
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='EnvironmentPermission']"
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Comment" "//SecurityClasses/SecurityClass[@Name='PrintingPermission']"
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Add" "//SecurityClasses" "SecurityClass" @( ("Name","OleDbPermission"), ("Description","System.Data.OleDb.OleDbPermission, System.Data, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089") )
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='EnvironmentPermission']"
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Comment" "//NamedPermissionSets/PermissionSet/IPermission[@class='PrintingPermission']"
		ModifyXML "$dNET4_64_dir\cust_web_mediumtrust.config" "Add" "//PermissionSet[@Name='ASP.Net']" "IPermission" @( ("class","OleDbPermission"), ("version","1"), ("Unrestricted", "true") )
		ModifyXML "$dNET4_64_dir\web.config" "Update" "//location[@allowOverride='true']/@allowOverride" "false"
		ModifyXML "$dNET4_64_dir\web.config" "Add" "//system.web/securityPolicy" "trustLevel" @( ("name","Custom"), ("policyFile", "cust_web_mediumtrust.config") )
	}
}


####################################################################################################################################################################################
Function dEntSvrQuerySQL ($dQueryString)    # Function to query the Enterprise Server Database directly
{ # Usage - (dEntSvrQuerySQL "[Query String]").ValueRequired
	$SqlConnection = New-Object System.Data.SqlClient.SqlConnection
	$SqlConnection.ConnectionString = (( ([xml](Get-Content ( "\\" + ($dSolidCPEnterpriseSvrIP) + "\c$\SolidCP\Enterprise Server\Web.config" ) )).configuration.connectionStrings.add.connectionString) -replace "localhost", $dSolidCPEnterpriseSvrIP)
	$SqlConnection.Open()
	$SqlCmd = New-Object System.Data.SqlClient.SqlCommand
	$SqlCmd.CommandText = $dQueryString
	$SqlCmd.Connection = $SqlConnection
	$SqlDataset = New-Object System.Data.DataSet
	(New-Object System.Data.sqlclient.sqlDataAdapter $SqlCmd).Fill($SqlDataset) | Out-Null
	$SqlConnection.Close()
	$SqlDataset.Tables
}
#(dEntSvrQuerySQL "SELECT [Username], [Password] FROM [Users] WHERE [Username] = 'SCPWebDavPeer'").Password


####################################################################################################################################################################################
Function dSQLAddWebDavUser ($dSCPwebDavUser, $dSCPwebDavPassword) # Function to inset a new user into the Database - *** Testing ONLY ***
{ # Usage - dSQLAddWebDavUser "[UserName]" "[EncryptecPassword]"
$dQueryString = @"
	IF NOT EXISTS (SELECT [Username] FROM [Users] WHERE [Username] = '$dSCPwebDavUser')
	BEGIN
		INSERT INTO [Users] ( [OwnerID], [RoleID], [StatusID], [IsDemo], [IsPeer], [Username], [Password], [FirstName], [LastName], [Created], [Changed], [HtmlMail], [EcommerceEnabled], [LoginStatusId] )
		values (1, 1, 1, 0, 1, '$dSCPwebDavUser', 'QcT3bD17g9PpaNg1vPj2bNcd7DaNbuAs8MkD4giqPX4=', 'SolidCP', 'WebDav Peer Account', CURRENT_TIMESTAMP, CURRENT_TIMESTAMP, 0, 0, 0)
	END
"@
	dEntSvrQuerySQL $dQueryString
}


####################################################################################################################################################################################
Function InstallSOlidCPInstaller()          # Function to download and install the latest SolidCP Installer from the SolidCP Website
{
	if (!(Test-Path "C:\Program Files (x86)\SolidCP Installer")) {
		# Create the SolidCP Directory in our Installation Files folder ready for downloading if it doesn't already exist
		if (!(Test-Path 'C:\_Install Files\SolidCP')) { (md -Path 'C:\_Install Files\SolidCP' -Force) | Out-Null }
		# Get the highest current SolidCP Version from the Installer XML File from the SolidCP Installation Website
		[string]$dSolidCPversion = ((([xml](New-Object System.Net.WebClient).DownloadString("http://installer.solidcp.com/Data/ProductReleasesFeed-1.0.xml")).components.component.releases.release).version | measure -Maximum).Maximum
		# Download the latest SolidCP Installer and save it to the "C:\_Install Files\SolidCP\" directory
		Write-Host "`tDownloading the latest SolidCP Installer" -ForegroundColor Cyan
		(New-Object System.Net.WebClient).DownloadFile("http://installer.solidcp.com/Files/$dSolidCPversion/SolidCPInstaller.msi", "C:\_Install Files\SolidCP\SolidCPInstaller-v$dSolidCPversion.msi")
		# Install the SolidCP Installer on this machine
		Write-Host "`tInstalling the latest SolidCP Installer on this machine" -ForegroundColor Cyan
		((Start-Process -FilePath "C:\_Install Files\SolidCP\SolidCPInstaller-v$dSolidCPversion.msi" -ArgumentList "/qb IACCEPTSQLNCLILICENSETERMS=YES" -Wait -Passthru).ExitCode) | Out-Null
	}
}


####################################################################################################################################################################################
Function InstallSOlidCPcomponent($dComponent, $dComponentPassword)  # Function to install the required SolidCP Component from the SolidCP Installer
{# Usage - InstallSOlidCPcomponent Enterprise|Portal|Server|WebDav [Password for component]
	######################################################################################################################
	# Variables - these are all of the variables that can be passed to the SolidCP.SilentInstaller.exe file
	#
	# webip     # General - The IP Address that you want the SolidCP Component to be bound to in IIS
	# webport   # General - The Port Number that you want the SolidCP Component to be bound to in IIS
	# webdom    # General - The FQDN that you want the SolidCP Component to be bound to in IIS
	# uname     # General - The Service Name account that you want to use for your SolidCP deployment
	# upassw    # General - The Service Name password that you want to use for your SolidCP deployment
	# udomaim   # General - The Domain Name that you want to use for your SolidCP deployment - *** If this is used then the above username and password will be added to Active Directory ***
	# passw     # Server Password - Used on the Server and Enterprise Server Components ONLY
	# dbname    # Enterprise Server - Database Name
	# dbserver  # Enterprise Server - Database Server (IP or Hostname)
	# dbadmin   # Enterprise Server - Database Username
	# dbapassw  # Enterprise Server - Database Password
	# esurl     # Portal Server - Enterprise Server URL - Used on the Portal Installation Component ONLY
	# cname     # "Server"|"Enterprise Server"|"Portal"|"WebDavPortal" - This is the component you want to install
	#
	# Install the SolidCP Server component from the SolidCP.SilentInstaller.exe file with the corasponding arguments above
	######################################################################################################################
	if (Test-Path "C:\Program Files (x86)\SolidCP Installer") {
		if ($dComponent -eq "Enterprise") {   # Install the SolidCP Enterprise Server component
			if (!(Test-Path "C:\SolidCP\Enterprise Server")) { # Check to see if it is already installed, if not then install it
				if ($dComponentPassword) {$dSolidCPportalPassword = $dComponentPassword} # set the password if it has been specified as part of the function
				if (!($dSolidCPportalPassword)) { # Ask the user to enter the password they want to for the WebDav component (for communication between the Server and the Enterprise Server)
					$dSolidCPportalPassword = Read-Host "Please enter the password you want to use for the SolidCP Portal Component (Web GUI ServerAdmin account)"
				}
				Write-Host "`tDownloading and Installing the SolidCP `"Enterprise Server`" component" -ForegroundColor Cyan
				if ($dDomainMember) { # If this server is joined to a domain then install the SolidCP Enterprise Server component Service Account in Active Directory
					$dRandomPasswordEntSvr = [guid]::NewGuid() # Generate a random password for the Domain SolidCP Enterprise Server Service Account
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Enterprise Server`" /passw:`"$dSolidCPportalPassword`" /webip:`"*`" /webport:`"9002`" /udomaim:`"$env:USERDNSDOMAIN`" /uname:`"SCPEnterprise`" /upassw:`"$dRandomPasswordEntSvr`"" -Wait -Passthru).ExitCode | Out-Null
				}else{ # If the server is not joined to a domain then install the SolidCP Enterprise Server component normally
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Enterprise Server`" /passw:`"$dSolidCPportalPassword`" /webip:`"*`" /webport:`"9002`"" -Wait -Passthru).ExitCode | Out-Null
				}
			}
		}elseif ($dComponent -eq "Portal") {  # Install the SolidCP Portal component
			if (!(Test-Path "C:\SolidCP\Portal")) { # Check to see if it is already installed, if not then install it
				if (!($dSolidCPEnterpriseSvrURL)) { # Chec to see if the Enterpeise Server URL is defined
					if ( (Test-Path "C:\SolidCP\Enterprise Server") -and (Get-WebBinding -Name 'SolidCP Enterprise Server') ) { # Check to see if the Enterprise Server is installed on this machine
						# Find the SolidCP Enterprise Server URL and port so that it can be used in the portal installation only if it
						if (((Get-WebBinding -Name 'SolidCP Enterprise Server').bindingInformation).substring(((Get-WebBinding -Name 'SolidCP Enterprise Server').bindingInformation).length -1,1) -eq "`:") {
							$dSolidCPEnterpriseSvrURL = ((Get-WebBinding -Name 'SolidCP Enterprise Server').protocol) + "://" + ((((Get-WebBinding -Name 'SolidCP Enterprise Server').bindingInformation).substring(0,((Get-WebBinding -Name 'SolidCP Enterprise Server').bindingInformation).length -1) -replace '[*]','127.0.0.1') -replace '[\[\]]','')
						}else{
							$dSolidCPEnterpriseSvrURL = ((Get-WebBinding -Name 'SolidCP Enterprise Server').protocol) + "://" + ((((Get-WebBinding -Name 'SolidCP Enterprise Server').bindingInformation) -replace '[*]','127.0.0.1') -replace '[\[\]]','')
						}
					}else{ # Otherwise ask for the Enterprise Server URL
							$dSolidCPEnterpriseSvrURL = Read-Host "Please enter the SolidCP Enterprise Server URL (i.e. http://127.0.0.1:9002)"
					}
				}
				Write-Host "`tDownloading and Installing the SolidCP `"Portal`" component" -ForegroundColor Cyan
				if ($dDomainMember) { # If this server is joined to a domain then install the SolidCP Portal component Service Account in Active Directory
					$dRandomPasswordPortal = [guid]::NewGuid() # Generate a random password for the Domain SolidCP Portal Service Account
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Portal`" /webip:`"$dSolidCPportalIPaddress`" /webport:`"$dSolidCPportalPortNumber`" /esurl:`"$dSolidCPEnterpriseSvrURL`" /udomaim:`"$env:USERDNSDOMAIN`" /uname:`"SCPPortal`" /upassw:`"$dRandomPasswordPortal`"" -Wait -Passthru).ExitCode | Out-Null
				}else{ # If the server is not joined to a domain then install the SolidCP Portal component normally
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Portal`" /webip:`"$dSolidCPportalIPaddress`" /webport:`"$dSolidCPportalPortNumber`" /esurl:`"$dSolidCPEnterpriseSvrURL`"" -Wait -Passthru).ExitCode | Out-Null
				}
				# Set the correct icon for the Desktop shortcut
				if ( (Test-Path "$env:USERPROFILE\Desktop\Login to SolidCP.url") -and (Test-Path "C:\SolidCP\Portal\favicon.ico") ) {
					($objShortcut = ((((New-Object -comObject Shell.Application).NameSpace(0X0)).ParseName("$env:USERPROFILE\Desktop\Login to SolidCP.url")).GetLink)) | Out-Null
					($objShortcut.SetIconLocation("C:\SolidCP\Portal\favicon.ico",0)) | Out-Null
					($objShortcut.Save()) | Out-Null
				}
				# Set the correct icon for the Start Menu shortcut
				if ( (Test-Path "$env:APPDATA\Microsoft\Windows\Start Menu\Programs\SolidCP Software\Login to SolidCP.url") -and (Test-Path "C:\SolidCP\Portal\favicon.ico") ) {
					($objShortcut = ((((New-Object -comObject Shell.Application).NameSpace(0X0)).ParseName("$env:APPDATA\Microsoft\Windows\Start Menu\Programs\SolidCP Software\Login to SolidCP.url")).GetLink)) | Out-Null
					($objShortcut.SetIconLocation("C:\SolidCP\Portal\favicon.ico",0)) | Out-Null
					($objShortcut.Save()) | Out-Null
				}
			}
		}elseif ($dComponent -eq "Server") {  # Install the SolidCP Server component
			if (!(Test-Path "C:\SolidCP\Server")) { # Check to see if it is already installed, if not then install it
				Write-Host "`tDownloading and Installing the SolidCP `"Server`" component" -ForegroundColor Cyan
				if ($dComponentPassword) {$dSolidCPserverPassword = $dComponentPassword} # set the password if it has been specified as part of the function
				if (!($dSolidCPserverPassword)) { # Ask the user to enter the password they want to for the WebDav component (for communication between the Server and the Enterprise Server)
					$dSolidCPserverPassword = Read-Host "Please enter the password you want to use for the SolidCP Server Component"
				}
				if ($dDomainMember) { # If this server is joined to a domain then install the SolidCP Server component Service Account in Active Directory
					$dRandomPasswordServer = [guid]::NewGuid() # Generate a random password for the Domain SolidCP Server Service Account
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Server`" /passw:`"$dSolidCPserverPassword`" /webip:`"$dIPV4`" /webport:`"9003`" /udomaim:`"$env:USERDNSDOMAIN`" /uname:`"SCPServer-$env:computerName`" /upassw:`"$dRandomPasswordServer`"" -Wait -Passthru).ExitCode | Out-Null
					(AddDomainUserToLocalGroup "Administrators File Access" "SCPServer-$env:computerName") | Out-Null  # Add the "SCPServer-[ComputerName]" User to the new group called "Administrators File Access"
					(AddDomainUserToDomainGroup "$dLangDomainAdminsGroup" "SCPServer-$env:computerName") | Out-Null    # Add the "SCPServer-[ComputerName]" User to the group called "Domain Admins"
				}else{ # If the server is not joined to a domain then install the SolidCP Server component normally
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"Server`" /passw:`"$dSolidCPserverPassword`" /webip:`"$dIPV4`" /webport:`"9003`"" -Wait -Passthru).ExitCode | Out-Null
					(AddLocalUserToLocalGroup "Administrators File Access" "SCPServer") | Out-Null                     # Add the "SCPServer" User to the new group called "Administrators File Access"
				}
			}
		}elseif ($dComponent -eq "WebDav") {  # Install the SolidCP WebDav (Cloud Storage Portal) component
			if (!(Test-Path "C:\SolidCP\Cloud Storage Portal")) { # Check to see if it is already installed, if not then install it
				if (!($dSolidCPEnterpriseSvrURL)) { # Check if the Enterprise Server URL has been set, if not ask for it
					$dSolidCPEnterpriseSvrURL = Read-Host "Please enter the SolidCP Enterprise Server URL (i.e. http://PublicIP:9002)"
				}
				if (!($dSolidCPEnterpriseSvrIP)) { # Check if the Enterprise Server IP has been set, if not ask for it
					$dSolidCPEnterpriseSvrIP = Read-Host "Please enter the SolidCP Enterprise Server IP Address ONLY (i.e. 192.168.1.1)"
				}
				if (!($dSolidCPWebDavRootFQDN)) { # Ask the user to enter the FQDN of the WebDav website they want to create (unless it has been specified in user variables
					$dSolidCPWebDavRootFQDN = Read-Host "Please enter the FULL DNS Name you want to use for the SolidCP Cloud Storage Portal (i.e. http://filesvr.solidcp.com)"
				}
				if ($dComponentPassword) {$dSolidCPCloudPortlPasswd = $dComponentPassword} # set the password if it has been specified as part of the function
				if (!($dSolidCPCloudPortlPasswd)) { # Ask the user to enter the password they want to for the WebDav component (for communication between the Server and the Enterprise Server)
					$dSolidCPCloudPortlPasswd = Read-Host "Please enter the password you want to use for the SolidCP Cloud Storage Portal Component"
				}
				Write-Host "`tDownloading and Installing the SolidCP `"Cloud Storage Portal`" component" -ForegroundColor Cyan
				if ($dDomainMember) { # If this server is joined to a domain then install the SolidCP WebDav (Cloud Storage Portal) component Service Account in Active Directory
					$dRandomPasswordWebDav = [guid]::NewGuid() # Generate a random password for the Domain SolidCP WebDav Service Account
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"WebDavPortal`" /passw:`"$dSolidCPCloudPortlPasswd`" /webip:`"$dIPV4`" /webport:`"9004`" /esurl:`"http://$dSolidCPEnterpriseSvrURL`" /udomaim:`"$env:USERDNSDOMAIN`" /uname:`"SCPWebDav-$env:computerName`" /upassw:`"$dRandomPasswordWebDav`"" -Wait -Passthru).ExitCode | Out-Null
					(AddDomainUserToLocalGroup "Administrators File Access" "SCPWebDav-$env:computerName") | Out-Null  # Add the "SCPWebDav-[ComputerName]" User to the new group called "Administrators File Access"
					(AddDomainUserToDomainGroup "$dLangDomainAdminsGroup" "SCPWebDav-$env:computerName") | Out-Null    # Add the "SCPWebDav-[ComputerName]" User to the group "Domain Admins"
				}else{ # If the server is not joined to a domain then install the SolidCP SCPWebDav (Cloud Storage Portal) component normally
					(Start-Process -FilePath 'C:\Program Files (x86)\SolidCP Installer\SolidCP.SilentInstaller.exe' -Argumentlist "/cname:`"WebDavPortal`" /passw:`"$dSolidCPCloudPortlPasswd`" /webip:`"$dIPV4`" /webport:`"9004`" /esurl:`"http://$dSolidCPEnterpriseSvrURL`"" -Wait -Passthru).ExitCode | Out-Null
					(AddLocalUserToLocalGroup "Administrators File Access" "SCPWebDav") | Out-Null                     # Add the "SCPWebDav" User to the new group called "Administrators File Access"
				}
				# Check if the SolidCP Cloud Storage Portal web.config file is there, if it is then modify it as below
				if (Test-Path "C:\SolidCP\Cloud Storage Portal\Web.config") {
					# Set the values in the web.config file for the Cloud Storage Portal (WebDav)
					[xml]$myXML = Get-Content "C:\SolidCP\Cloud Storage Portal\Web.config"
					($myXML.configuration.appSettings.add | ? { $_.key -eq "SolidCP.CryptoKey" }).value = (([xml](Get-Content ( "\\" + ($dSolidCPEnterpriseSvrIP) + "\c$\SolidCP\Enterprise Server\Web.config" ) )).configuration.appSettings.add | ? { $_.key -eq "SolidCP.CryptoKey" }).value
					$myXML.configuration.webDavExplorerConfigurationSettings.userDomain.value= $env:USERDNSDOMAIN.ToLower()
					$myXML.configuration.webDavExplorerConfigurationSettings.webdavRoot.value = $dSolidCPWebDavRootFQDN + "/"
					$myXML.configuration.webDavExplorerConfigurationSettings.enterpriseServer.url = $dSolidCPEnterpriseSvrURL
					$myXML.configuration.webDavExplorerConfigurationSettings.SolidCPConstantUser.login = "serveradmin"
					$myXML.configuration.webDavExplorerConfigurationSettings.SolidCPConstantUser.password = (dEntSvrQuerySQL "SELECT [Username], [Password] FROM [Users] WHERE [Username] = 'serveradmin'").Password
					$myXML.Save("C:\SolidCP\Cloud Storage Portal\Web.config")
				}
				# Make the required changes ONLY if the SolidCP Cloud Storage Portal site exists
				if (Test-Path "IIS:\Sites\SolidCP Cloud Storage Portal" -pathType container) {
					# Get the Application Pool for the SolidCP Cloud Storage Portal ready to make changes
					$pool = Get-Item 'IIS:\AppPools\SolidCP Cloud Storage Portal Pool'
					$pool.autoStart = 'True'                                # Enable Auto-Start
					$pool.startMode = 'AlwaysRunning'                       # Set Start Mode as Always Running
					$pool.processModel.idleTimeout = [TimeSpan]::Zero       # Set Idele TimeOut as 0 Minutes
					$pool.recycling.periodicRestart.time = [TimeSpan]::Zero # Set Regular time interval as 0 Minutes
					$pool | Set-Item                                        # Set all of the above for the SolidCP Cloud Storage Portal Application Pool
					# Remove any schedules for the SolidCP Cloud Storage Portal Application Pool and set the time to be 3.00am as it needs to be recycled once a day
					Clear-ItemProperty "IIS:\AppPools\SolidCP Cloud Storage Portal Pool" -Name recycling.periodicRestart.schedule
					Set-ItemProperty "IIS:\AppPools\SolidCP Cloud Storage Portal Pool" -Name recycling.periodicRestart.schedule -Value @{value='03:00:00'}
					# Disable Anonymous Authentication on the SolidCP Cloud Storage Portal website
					#Get-WebConfiguration "system.webServer/security/authentication/anonymousAuthentication" -PSPath IIS:\ -Location "SolidCP Cloud Storage Portal" -Value @{enabled="False"}
					# Disable Windows Authentication on the SolidCP Cloud Storage Portal website
					Set-WebConfiguration system.webServer/security/authentication/windowsAuthentication -PSPath IIS:\ -Location "SolidCP Cloud Storage Portal" -Value @{enabled="False"}
				}
			}
		}
	}
}


####################################################################################################################################################################################
####################################################################################################################################################################################
# Run the SolidCP Installation Menu as long as the logged in user is member of the Local "Administrators" group of the "Domain Admins" group
if (!($dDomainMember)) { # Check to see if the machine is NOT joined to a domain
	if (CheckGroupMembers "$dLangAdministratorGroup" "$dLoggedInUserName" "Local") { # Run the SOlidCP Menu if the logged in user is a Local Administrator
		Write-Host "`n`t This machine is NOT Joined to domain and you are logged in as Local Administrator Account" -ForegroundColor Green
		Write-Host "`t The SolidCP menu is being loaded" -ForegroundColor Green
		SolidCPmenu
	}elseif (!(CheckGroupMembers "$dLangAdministratorGroup" "$dLoggedInUserName" "Local")) { # The logged in user is NOT a Local Administrator
		Write-Host "`n`t *************************************************" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *         You dont seem to be logged in         *" -ForegroundColor Yellow
		Write-Host "`t *         with an Administrative account        *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *   Please log back on with an account that is  *" -ForegroundColor Yellow
		Write-Host "`t *      a member of the Administrators group     *" -ForegroundColor Yellow
		Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *************************************************" -ForegroundColor Yellow
		dPressAnyKeyToExit
	}
}elseif ( ($dDomainMember) -and ($dLoggedInLocally) ) {
	Write-Host "`n`t *************************************************" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *     This machine is a member of a domain      *" -ForegroundColor Yellow
	Write-Host "`t *  and you are legged in with a local account   *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *    You need to login with a domain account    *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *  Please log back on with an account that is   *" -ForegroundColor Yellow
	Write-Host "`t *      a member of the Domain Admins group      *" -ForegroundColor Yellow
	Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *************************************************" -ForegroundColor Yellow
	dPressAnyKeyToExit
}elseif ( ($dDomainMember) -and (!($dLoggedInLocally)) ) {
	if (CheckGroupMembers "$dLangDomainAdminsGroup" "$dLoggedInUserName" "Domain") { # Run the SOlidCP Menu if the logged in user is a Domain Administrator
		Write-Host "`n`t This machine is Joined to domain and you are logged in as Domain Administrator Account" -ForegroundColor Green
		Write-Host "`t The SolidCP menu is being loaded" -ForegroundColor Green
		SolidCPmenu
	}elseif (!(CheckGroupMembers "$dLangDomainAdminsGroup" "$dLoggedInUserName" "Domain")) { # The logged in user is NOT a Domain Administrator
		Write-Host "`n`t *************************************************" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *     This machine is a member of a domain      *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *    You need to login with a domain account    *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *  Please log back on with an account that is   *" -ForegroundColor Yellow
		Write-Host "`t *      a member of the Domain Admins group      *" -ForegroundColor Yellow
		Write-Host "`t *           and run this script again           *" -ForegroundColor Yellow
		Write-Host "`t *                                               *" -ForegroundColor Yellow
		Write-Host "`t *************************************************" -ForegroundColor Yellow
		dPressAnyKeyToExit
	}
}else{ # Show this error if the above conditions are not met, it should never apear
	CLS
	Write-Host "`n`t *************************************************" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *     Oops, An unexpected error has occurred    *" -ForegroundColor Yellow
	Write-Host "`t *      We apologize for this inconvenience.     *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *    Please contact SolidCP Technical Support   *" -ForegroundColor Yellow
	Write-Host "`t *            on support@solidcp.com             *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *      Please let them know the error was       *" -ForegroundColor Yellow
	Write-Host "`t *      with the SolidCP PowerShell Script       *" -ForegroundColor Yellow
	Write-Host "`t *                                               *" -ForegroundColor Yellow
	Write-Host "`t *************************************************" -ForegroundColor Yellow
	Write-Host "`n`n`t ==============  DEBUG INFORMATION  ==============" -ForegroundColor Green
	Write-Host "`t Joined to domain     = $dDomainMember" -ForegroundColor Green
	Write-Host "`t FQDN of this machine = $dFQDNthisMachine" -ForegroundColor Green
	Write-Host "`t Domain Name          = $dDomainName" -ForegroundColor Green
	Write-Host "`t Computer Name        = $dComputerName" -ForegroundColor Green
	Write-Host "`t Logged in User Name  = $dLoggedInUserName" -ForegroundColor Green
	Write-Host "`t Logged in Locally    = $dLoggedInLocally" -ForegroundColor Green
	dPressAnyKeyToExit
}

